SystemOrganization addCategory: #'Sight-Dynamic'!
SystemOrganization addCategory: #'Sight-Dynamic-UI'!
SystemOrganization addCategory: #'Sight-Dynamic-Models'!
SystemOrganization addCategory: #'Sight-Dynamic-Testbed'!
SystemOrganization addCategory: #'Sight-Dynamic-Events'!
SystemOrganization addCategory: #'Sight-Dynamic-Tests'!

UiTestCase subclass: #SdMethodFinderTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sight-Dynamic-Tests'!

!SdMethodFinderTests methodsFor: 'tests' stamp: 'mt 4/12/2011 12:55:20.495'!
test01Resizing

	| m |
	m := Morph new
		changeTableLayout;
		hResizing: #spaceFill;
		vResizing: #shrinkWrap.
	sut
		theClass: String.
	
	m addMorph: sut.
	self assert: sut height equals: m height.! !

!SdMethodFinderTests methodsFor: 'tests' stamp: 'mt 4/12/2011 12:59:27.046'!
test02New

	sut cellInset: 0.  
	self assert: sut ui searchField height equals: sut height.! !

UiPushButton subclass: #SdResizeHandle
	instanceVariableNames: 'flattened fadeAnimation mouseDownEventPosition mouseDownTargetExtent target orientation'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sight-Dynamic'!

!SdResizeHandle commentStamp: 'mt 4/13/2011 13:23:38.993' prior: 0!
A handle to resize morphs in width.!

!SdResizeHandle class methodsFor: 'testing' stamp: 'mt 4/14/2011 10:35:12.804'!
isWidgetClass

	^ false! !

!SdResizeHandle methodsFor: 'initialization' stamp: 'mt 4/15/2011 10:02:21.155'!
addGrips

	| container |
	container := Morph new
		color: Color transparent;
		changeTableLayout;
		listDirection: (self orientation = #vertical ifTrue: [#topToBottom] ifFalse: [#leftToRight]);  
		vResizing: (self orientation = #vertical ifTrue: [#spaceFill] ifFalse: [#shrinkWrap]);
		hResizing: (self orientation = #vertical ifTrue: [#shrinkWrap] ifFalse: [#spaceFill]);
		cellInset: 1;
		addMorphBack: (UiSpacer new orientation: self orientation; extent: 5@5).
		  
	4 timesRepeat: [container addMorphBack: (
		Morph new
			extent: 3@3;
			color: (Color gray: 0.7);
			borderStyle: (BorderStyle raised width: 1);
			ignoresColorMappings: true)].
		
	container addMorphBack: (UiSpacer new orientation: self orientation; extent: 5@5).
	
	self
		removeAllMorphs;
		addMorphBack: container.  ! !

!SdResizeHandle methodsFor: 'actions' stamp: 'mt 4/15/2011 10:27:39.249'!
doHandleMovement: aMouseMoveEvent

	self target ifNil: [^ self].
	
	self orientation = #vertical ifTrue: [
		self target width:
			((mouseDownTargetExtent
				+ aMouseMoveEvent position
				- mouseDownEventPosition) x
			max: 50)].
		
	self orientation = #horizontal ifTrue: [
		self target height:
			((mouseDownTargetExtent
				+ aMouseMoveEvent position
				- mouseDownEventPosition) y
			max: 50)].! !

!SdResizeHandle methodsFor: 'actions' stamp: 'mt 4/13/2011 13:02:53.838'!
fadeIn

	self visible ifFalse: [^ self].
	
	fadeAnimation
		ifNil: [^ self]
		ifNotNil: [:anim |
			anim unregister.
			fadeAnimation := nil].
	
	super fadeIn.! !

!SdResizeHandle methodsFor: 'actions' stamp: 'mt 4/13/2011 13:19:26.159'!
fadeOut

	self visible ifFalse: [^ self].
	mouseDownEventPosition ifNotNil: [^ self].
	
	fadeAnimation ifNil: [fadeAnimation := (AnimAlphaBlendAnimation
		fadeOutOn: self
		duration: 200)
		register;
		start: #keepWhenFinished].! !

!SdResizeHandle methodsFor: 'actions' stamp: 'mt 4/15/2011 10:17:02.852'!
finishHandleMovement: aMouseEvent

	mouseDownEventPosition := nil.
	mouseDownTargetExtent := nil.
	
	(self bounds containsPoint: aMouseEvent position)
		ifFalse: [self fadeOut].! !

!SdResizeHandle methodsFor: 'initialization' stamp: 'mt 4/15/2011 10:26:09.106'!
initialize

	orientation := #vertical.

	super initialize.
	
	self
		extent: 9@10;
		color: (Color gray: 0.85);
		listCentering: #center;
		layoutInset: 0;
		updateLayout;
		fadeOut.
		
	self eventHandler setProperty: #handlesKeyboard toValue: [false].

	self connect: self eventHandler signal: #mouseEntered: toSelector: #fadeIn.
	self connect: self eventHandler signal: #mouseLeft: toSelector: #fadeOut.
	
	self connect: self eventHandler signal: #mousePressed: toSelector: #startHandleMovement:.
	self connect: self eventHandler signal: #mouseMoved: toSelector: #doHandleMovement:.
	self connect: self eventHandler signal: #mouseReleased: toSelector: #finishHandleMovement:.
	
	self connect: self eventHandler signal: #mouseClicked: toSelector: #onClick.
	
	self eventHandler disconnectSignal: #mousePressed: from: self selector: #grabKeyboard.! !

!SdResizeHandle methodsFor: 'actions' stamp: 'mt 4/15/2011 10:24:26.597'!
onClick

	self target ifNil: [^ self].
	
	self orientation = #vertical
		ifTrue: [self target width: 500].! !

!SdResizeHandle methodsFor: 'accessing' stamp: 'mt 4/15/2011 09:50:35.156'!
orientation

	^ orientation! !

!SdResizeHandle methodsFor: 'accessing' stamp: 'mt 4/15/2011 09:55:12.167'!
orientation: aSymbol
	"#horizontal, #vertical"

	orientation = aSymbol ifTrue: [^ self].

	orientation := aSymbol.
	
	self
		extent: self height @ self width; "flip"
		updateLayout.! !

!SdResizeHandle methodsFor: 'actions' stamp: 'mt 4/15/2011 10:16:55.785'!
startHandleMovement: aMouseEvent

	self target ifNil: [^ self].

	mouseDownEventPosition := aMouseEvent position.
	mouseDownTargetExtent := self target extent.! !

!SdResizeHandle methodsFor: 'accessing' stamp: 'mt 4/15/2011 09:48:53.381'!
target
	"Morph to be resized."
	
	^ target! !

!SdResizeHandle methodsFor: 'accessing' stamp: 'mt 4/15/2011 09:48:08.132'!
target: aMorph

	target := aMorph.! !

!SdResizeHandle methodsFor: 'updating' stamp: 'mt 4/15/2011 09:56:33.897'!
updateLayout

	self removeAllMorphs.
	
	self orientation = #horizontal ifTrue: [
		self
			hResizing: #spaceFill;
			vResizing: #rigid;
			listDirection: #topToBottom].
	
	self orientation = #vertical ifTrue: [
		self
			hResizing: #rigid;
			vResizing: #spaceFill;
			listDirection: #leftToRight].

	self addGrips.! !

UiTextEdit subclass: #SdSourceCodeEdit
	instanceVariableNames: 'theClass selector protocolBtn sideBtn'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sight-Dynamic'!

!SdSourceCodeEdit commentStamp: 'mt 3/24/2011 11:22:15.769' prior: 0!
This is a text edit to modify a method's source.!

SdSourceCodeEdit subclass: #SdClassDefinitionEdit
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sight-Dynamic'!

!SdClassDefinitionEdit methodsFor: 'actions' stamp: 'mt 4/12/2011 15:40:00.775'!
compile
	"No classic compile but code evaluation."
	
	Compiler evaluate: self theText asString.
	
	^ true! !

!SdClassDefinitionEdit methodsFor: 'initialization' stamp: 'mt 4/18/2011 16:25:07.137'!
initialize

	super initialize.
	
	protocolBtn delete.
	sideBtn delete.! !

!SdClassDefinitionEdit methodsFor: 'actions' stamp: 'mt 4/12/2011 15:39:54.224'!
parse

	^ true! !

!SdClassDefinitionEdit methodsFor: 'initialization' stamp: 'mt 4/14/2011 15:16:40.77'!
reset

	self theClass ifNotNil: [:c |
		self textMorph styler classOrMetaClass: c].! !

!SdSourceCodeEdit class methodsFor: 'testing' stamp: 'mt 3/25/2011 17:07:38.3'!
isWidgetClass

	^ false! !

!SdSourceCodeEdit methodsFor: 'actions' stamp: 'mt 4/11/2011 10:17:38.635'!
acceptChanges

	self hasUnacceptedChanges
		ifFalse: [^ self].
		
	(self parse and: [self compile])
		ifTrue: [^ super acceptChanges].
		
	self grabKeyboard.! !

!SdSourceCodeEdit methodsFor: 'actions' stamp: 'mt 4/14/2011 16:58:36.182'!
changeProtocol: newProtocolName

	self theClass organization
		classify: self selector
		under: newProtocolName
		suppressIfDefault: false.! !

!SdSourceCodeEdit methodsFor: 'actions' stamp: 'mt 4/11/2011 09:44:55.408'!
checkSelector
	"Check current selector and reload source if necessary."
	
	self selector: (Compiler parserClass new parseSelector: self theText asString).! !

!SdSourceCodeEdit methodsFor: 'actions' stamp: 'mt 4/11/2011 09:45:33.703'!
compile

	[self theClass
		compile: self theText
		classified: (MethodReference
			class: self theClass
			selector: self selector) category.
			
	self checkSelector.
	
	^ true]
		on: SyntaxErrorNotification "Manual catch to disable interactive compilation."
		do: [:err | 
			Transcript cr; show: '[Syntax] ', err asString.
			(SdMessageEvent
				text: 'Compilation failed: ', self theClass asString, ' >> ', self selector asString
				kind: #warning
				class: self theClass) trigger.
				
			self notify: err errorMessage at: err location in: err messageText.
			
			^ false]
		on: Error
		do: [:err |
			Transcript cr; show: '[Error] ', err asString.
			(SdMessageEvent
				text: 'Compilation error: ', self theClass asString, ' >> ', self selector asString
				kind: #error
				class: self theClass) trigger.
				
			^ false].! !

!SdSourceCodeEdit methodsFor: 'signals' stamp: 'mt 4/8/2011 11:18:33.265'!
focusChanged: aRectangle

	self emitAlways.! !

!SdSourceCodeEdit methodsFor: 'initialization' stamp: 'mt 4/18/2011 16:31:57.073'!
initialize

	protocolBtn := SdProtocolButton new.
	sideBtn := SdFadingButton new
		balloonText: 'class side method';
		color: Color green muchDarker;
		text: '<font color="#FFFFFF">c</font>' asHtmlText.
	
	super initialize.
	
	self textMorph font: Preferences standardCodeFont.
	self textMorph styler: SHTextStylerST80 new.
	
	self connect: self textMorph signal: #focusChanged: toSelector: #onFocusChanged:.

	self connect: self eventHandler signal: #mouseWheelUp: toSelector: #mouseWheelUp:.
	self connect: self eventHandler signal: #mouseWheelDown: toSelector: #mouseWheelDown:.
	
	protocolBtn topRight: self transformMorph topRight.
	sideBtn topRight: protocolBtn bottomRight.
		
	self connect: protocolBtn signal: #protocolChanged: toSelector: #changeProtocol:.

	self
		addMorphFront: protocolBtn;
		addMorphFront: sideBtn.! !

!SdSourceCodeEdit methodsFor: 'signals' stamp: 'mt 4/8/2011 11:31:51.12'!
mouseWheelDown: aMouseEvent

	self emitAlways.! !

!SdSourceCodeEdit methodsFor: 'signals' stamp: 'mt 4/8/2011 11:31:41.523'!
mouseWheelUp: aMouseEvent

	self emitAlways.! !

!SdSourceCodeEdit methodsFor: 'actions' stamp: 'mt 4/10/2011 12:20:28.782'!
notify: aMessage at: aLocation in: aSource

	| message |
	self textMorph editor selectAt: aLocation.
	
	"Trigger manual focus changes. Should be automatic on selection changed?"
	self onFocusChanged: (self textMorph editor pointBlock).
	
	message := ((UiMessage information: aMessage)
		topLeft: (self textMorph localPointToGlobal: (self textMorph editor pointBlock bottomLeft))).
	message openInWorld.! !

!SdSourceCodeEdit methodsFor: 'focus' stamp: 'mt 4/8/2011 12:13:17.821'!
onFocusChanged: aLocalRectangle

	self focusChanged:
		((self textMorph localPointToGlobal: aLocalRectangle topLeft)
		extent: aLocalRectangle extent).! !

!SdSourceCodeEdit methodsFor: 'actions' stamp: 'mt 4/11/2011 10:34:19.586'!
parse

	| helper |
	helper := SdParserHelper new
		text: self theText;
		view: self.
	
	[Parser new
		parse: self theText asString readStream
		class: self theClass
		category: (MethodReference
			class: self theClass
			selector: self selector) category
		noPattern: false
		context: nil
		notifying: helper
		ifFail: [^ false]]
		on: ParserNotification "Manual catch to disable interactive compilation."
		do: [:err |
			(err isKindOf: UnusedVariable) ifTrue: [
				(SdMessageEvent
					text: 'Unused Variable found: ', self theClass asString, ' >> ', self selector asString
					kind: #information
					class: self theClass) trigger.

				err resume: false].
			
			(SdMessageEvent
				text: 'Parsing failed: ', self theClass asString, ' >> ', self selector asString
				kind: #warning
				class: self theClass) trigger.
				
			self notify: err class name at: helper selection first in: self theText.
			self select: helper selection.
			
			^ false].
		
	^ true! !

!SdSourceCodeEdit methodsFor: 'initialization' stamp: 'mt 4/18/2011 17:00:39.072'!
reset

	self theClass ifNil: [^ self].

	self textMorph styler classOrMetaClass: self theClass.  	
		
	self theClass isMeta
		ifTrue: [sideBtn show]
		ifFalse: [sideBtn hide].
	
	(self theClass includesSelector: self selector)
		ifTrue: [self hasUnacceptedChanges
			ifTrue: [self originalText: (self theClass >> self selector) getSource]
			ifFalse: [self theText: (self theClass >> self selector) getSource]]
		ifFalse: [self
			theText: self selector;
			originalText: '';
			"Save the new empty method."
			acceptChanges].
		
	protocolBtn
		organization: self theClass organization;
		text: (self theClass organization categoryOfElement: self selector).
		
	self updateButtonLayout.! !

!SdSourceCodeEdit methodsFor: 'accessing' stamp: 'mt 3/24/2011 11:01:21.227'!
selector

	^ selector! !

!SdSourceCodeEdit methodsFor: 'accessing' stamp: 'mt 3/25/2011 11:56:38.599'!
selector: aSymbol

	selector := aSymbol.! !

!SdSourceCodeEdit methodsFor: 'signals' stamp: 'mt 3/24/2011 11:37:39.359'!
sourceCodeResetted

	self emit.! !

!SdSourceCodeEdit methodsFor: 'accessing' stamp: 'mt 3/24/2011 11:01:05.836'!
theClass

	^ theClass! !

!SdSourceCodeEdit methodsFor: 'accessing' stamp: 'mt 3/25/2011 11:56:42.188'!
theClass: aClassOrMetaClass

	theClass := aClassOrMetaClass.! !

!SdSourceCodeEdit methodsFor: 'accessing' stamp: 'mt 3/25/2011 11:56:45.885'!
theClass: aClassOrMetaClass selector: aSymbol

	theClass := aClassOrMetaClass.
	selector := aSymbol.! !

!SdSourceCodeEdit methodsFor: 'layouting' stamp: 'mt 4/18/2011 17:00:18.707'!
updateButtonLayout
	
	protocolBtn
		fullBounds;
		topRight: self transformMorph topRight.
		
	sideBtn topRight: protocolBtn bottomRight.! !

!SdSourceCodeEdit methodsFor: 'layouting' stamp: 'mt 4/18/2011 17:00:27.011'!
updateLayout

	super updateLayout.
	
	self updateButtonLayout.! !

UiIcons subclass: #SdIcons
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sight-Dynamic'!

!SdIcons class methodsFor: 'accessing' stamp: 'mt 4/18/2011 13:51:34.853'!
classSideIcon
	
	^ Icons
		at: #classSide
		ifAbsentPut:[ Form fromBinaryStream: (Base64MimeConverter mimeDecodeToBytes: self classSideIconContents readStream) ].! !

!SdIcons class methodsFor: 'contents' stamp: 'mt 4/18/2011 13:50:50.089'!
classSideIconContents

	^ 'iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAAAXNSR0IArs4c6QAAAARnQU1B
AACxjwv8YQUAAAAJcEhZcwAADsMAAA7DAcdvqGQAAAAZdEVYdFNvZnR3YXJlAFBhaW50Lk5F
VCB2My41Ljg3O4BdAAABG0lEQVQ4T2NgoCmod9NgqHc1IM2OencFhjq3+UD8nqHe7T8c17kt
J2wYyDZ0jciGgNmuCdhdBHIuVDNHs/f/1oPL/n/4/uU/CIBoEJ+l0QPqIncHTEPAToQ4ee/d
82CN6KDryCqIAXVu51ENqLfnAAp+B0mazsyB6/NZWgvW4LG46v+rL+//r7l66D9Xiy/UFciB
C/I71PaUjX1wA1ACEW9Y1LtYUGiAlwTMAGQvgJwOEteakgL2ws7bZ/6LdIbAotYDNRzq3I4T
CsQN149CA9H1OQMo3FBAvVsAzABQNE46sR4lGkF8kDhUTQGOtODagD/gwFE4H3/SrneNYKgD
OREpGUPiHpi0gRYQDcBeArkIjCMw/Uy0SfgVAgD7EDtPjuUW4gAAAABJRU5ErkJggg=='! !

!SdIcons class methodsFor: 'accessing' stamp: 'mt 4/18/2011 13:51:50.81'!
instanceSideIcon
	
	^ Icons
		at: #instanceSide
		ifAbsentPut:[ Form fromBinaryStream: (Base64MimeConverter mimeDecodeToBytes: self instanceSideIconContents readStream) ].! !

!SdIcons class methodsFor: 'contents' stamp: 'mt 4/18/2011 13:50:55.035'!
instanceSideIconContents

	^ 'iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAAAXNSR0IArs4c6QAAAARnQU1B
AACxjwv8YQUAAAAJcEhZcwAADsMAAA7DAcdvqGQAAAAZdEVYdFNvZnR3YXJlAFBhaW50Lk5F
VCB2My41Ljg3O4BdAAAAyElEQVQ4T2NgoCmY9F+DYcp/A9LsmP5fgWHqv/kMU/69B2r+j8D/
lhM2DGQbhkZkQ8DsBOwuAjsX1db/UOC4Hs2Qyf8dMA2ZAnYikpNh2v//xzTg33lUA+b/52CY
/O870QZALEIKXLDfMfwKdwKGCyBqkcJi4j8LygyY8l+CdAP+eaCGw5R/x4kOg8n/njOAwg0F
TPofQIIBBdjTwuT/Ddi8giIGSqV4weR/EcAofY5pEDCRgSwgGoC8BNIAxkBDMfxMtEn4FQIA
y3l/8JebCsMAAAAASUVORK5CYII='! !

!SdIcons class methodsFor: 'license' stamp: 'mt 4/18/2011 13:49:52.176'!
license

	^ 'Composed or self-made icons by Marcel Taeumel (c) 2011.'! !

UiToolButton subclass: #SdFadingButton
	instanceVariableNames: 'fadeAnimation'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sight-Dynamic'!

!SdFadingButton methodsFor: 'animations' stamp: 'mt 4/18/2011 16:24:30.086'!
fadeIn

	self visible ifFalse: [^ self].

	fadeAnimation
		ifNil: [^ self]
		ifNotNil: [:anim |
			anim unregister.
			fadeAnimation := nil].
	
	(AnimAlphaBlendAnimation new
		morph: self;
		duration: 200;
		startValue: 0.3;
		endValue: 1.0;
		register;
		start: #deleteWhenFinished).! !

!SdFadingButton methodsFor: 'animations' stamp: 'mt 4/18/2011 16:24:25.999'!
fadeOut

	self visible ifFalse: [^ self].
	
	fadeAnimation ifNil: [fadeAnimation := (AnimAlphaBlendAnimation
		fadeOutOn: self
		to: 0.3
		duration: 200)
		register;
		start: #keepWhenFinished].! !

!SdFadingButton methodsFor: 'initialization' stamp: 'mt 4/18/2011 16:03:28.215'!
initialize

	super initialize.
	
	self
		fadeOut;
		layoutInset: (0@ -5 corner: 0@ -3).! !

!SdFadingButton methodsFor: 'initialization' stamp: 'mt 4/18/2011 15:56:09.335'!
initializeEventHandler

	super initializeEventHandler.
	
	self connect: self eventHandler signal: #mouseEntered: toSelector: #fadeIn.
	self connect: self eventHandler signal: #mouseLeft: toSelector: #fadeOut.! !

SdFadingButton subclass: #SdProtocolButton
	instanceVariableNames: 'organization'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sight-Dynamic'!

!SdProtocolButton class methodsFor: 'testing' stamp: 'mt 4/14/2011 15:02:09.464'!
isWidgetClass

	^ false! !

!SdProtocolButton methodsFor: 'initialization' stamp: 'mt 4/18/2011 16:03:39.71'!
initialize

	super initialize.
	
	self
		color: Color yellow paler;
		text: 'protocol';
		balloonText: 'method protocol'.! !

!SdProtocolButton methodsFor: 'initialization' stamp: 'mt 4/18/2011 15:56:15.093'!
initializeEventHandler

	super initializeEventHandler.
	
	self connect: self eventHandler signal: #mouseClicked: toSelector: #openProtocolList:.! !

!SdProtocolButton methodsFor: 'actions' stamp: 'mt 4/14/2011 16:14:07.233'!
openProtocolList: aMouseEvent

	| menu |
	menu := MenuMorph new
		defaultTarget: self.
		
	self organization categories
		select: [:category | category ~= self text asString asSymbol]
		thenDo: [:category | 
			menu
				add: category
				selector: #protocolChanged:
				argument: (category = #'as yet unclassified'
					ifTrue: []
					ifFalse: [category])].
		
	menu popUpForHand: aMouseEvent hand in: ActiveWorld.! !

!SdProtocolButton methodsFor: 'accessing' stamp: 'mt 4/14/2011 15:33:52.892'!
organization

	^ organization! !

!SdProtocolButton methodsFor: 'accessing' stamp: 'mt 4/14/2011 15:34:18.601'!
organization: anOrganizer

	organization := anOrganizer.! !

!SdProtocolButton methodsFor: 'signals' stamp: 'mt 4/14/2011 15:52:23.076'!
protocolChanged: newProtocolName

	self emitAlways.! !

Object subclass: #SdFoobar
	instanceVariableNames: 'b c'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sight-Dynamic-Testbed'!

SdFoobar class
	instanceVariableNames: 'a b'!

SdFoobar class
	instanceVariableNames: 'a b'!

!SdFoobar class methodsFor: 'instance creation' stamp: 'mt 4/18/2011 16:41:01.781'!
new

	^ super new! !

!SdFoobar class methodsFor: 'instance creation' stamp: 'mt 4/18/2011 16:29:24.929'!
newFor: me

	^ self new! !

!SdFoobar class methodsFor: 'as yet unclassified' stamp: 'mt 4/18/2011 17:02:45.559'!
ohMyGosh

	Transcript cr; show: b.! !

!SdFoobar class methodsFor: 'instance creation' stamp: 'mt 4/18/2011 16:49:00.002'!
woohooo! !

!SdFoobar methodsFor: 'initialization' stamp: 'mt 4/18/2011 16:46:38.831'!
ahah! !

!SdFoobar methodsFor: 'as yet unclassified' stamp: 'mt 4/18/2011 16:46:25.124'!
blubby

	"hmm."! !

!SdFoobar methodsFor: 'private' stamp: 'mt 4/14/2011 20:13:33.005'!
broom

	self halt.! !

!SdFoobar methodsFor: 'privateForMe' stamp: 'mt 4/14/2011 19:12:11.668'!
broom2! !

!SdFoobar methodsFor: 'privateForMe' stamp: 'mt 4/14/2011 19:17:30.211'!
helpMe! !

!SdFoobar methodsFor: 'privateForMe' stamp: 'mt 4/14/2011 20:14:07.289'!
iiii! !

!SdFoobar methodsFor: 'print' stamp: 'mt 4/17/2011 09:57:01.646'!
printString! !

!SdFoobar methodsFor: 'privateForMe' stamp: 'mt 4/18/2011 17:03:41.068'!
printString2! !

!SdFoobar methodsFor: 'initialization' stamp: 'mt 4/14/2011 19:28:14.756'!
pvtAddSugar

	self halt.
	

! !

!SdFoobar methodsFor: 'initialization' stamp: 'mt 4/14/2011 19:01:20.135'!
reset: aNumber! !

!SdFoobar methodsFor: 'initialization' stamp: 'mt 4/14/2011 19:07:06.999'!
resetMe! !

!SdFoobar methodsFor: 'initialization' stamp: 'mt 4/14/2011 19:08:45.346'!
resetMe: why! !

!SdFoobar methodsFor: 'private' stamp: 'mt 4/18/2011 16:43:50.194'!
wanye

	^ true! !

!SdFoobar methodsFor: 'privateForMe' stamp: 'mt 4/18/2011 16:48:49.156'!
woohooo

	"Jeht ja!!"! !

Object subclass: #SdFoobar10
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sight-Dynamic-Testbed'!

Object subclass: #SdFoobar11
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sight-Dynamic-Testbed'!

Object subclass: #SdFoobar2
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sight-Dynamic-Testbed'!

!SdFoobar2 methodsFor: 'as yet unclassified' stamp: 'mt 4/12/2011 14:48:26.137'!
foobar! !

Object subclass: #SdFoobar3
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sight-Dynamic-Testbed'!

!SdFoobar3 methodsFor: 'as yet unclassified' stamp: 'mt 4/12/2011 18:00:52.499'!
helloWorld

	! !

Object subclass: #SdFoobar4
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sight-Dynamic-Testbed'!

Object subclass: #SdFoobar5
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sight-Dynamic-Testbed'!

!SdFoobar5 commentStamp: '<historical>' prior: 0!
This class needs a comment.!

!SdFoobar5 methodsFor: 'as yet unclassified' stamp: 'mt 4/12/2011 18:24:49.345'!
a: anInteger b: aString

	Transcript cr; show: aString; cr; show: anInteger asString.! !

!SdFoobar5 methodsFor: 'as yet unclassified' stamp: 'mt 4/12/2011 18:23:31.988'!
firstMethod

	Transcript cr; show: 'Hello, World'.! !

Object subclass: #SdFoobar7
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sight-Dynamic-Testbed'!

!SdFoobar7 methodsFor: 'initialization' stamp: 'mt 4/17/2011 17:01:18.628'!
initialize! !

Object subclass: #SdFoobar8
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sight-Dynamic-Testbed'!

Object subclass: #SdParserHelper
	instanceVariableNames: 'text view selection'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sight-Dynamic'!

!SdParserHelper commentStamp: 'mt 4/11/2011 09:38:06.258' prior: 0!
Used for notification of default parser.!

!SdParserHelper methodsFor: 'interactive error handling' stamp: 'mt 4/13/2011 11:15:37.231'!
notify: aMessage at: aLocation in: aSource

	self view
		notify: (aMessage allButLast: 3) "remove trailing ' ->' "
		at: aLocation
		in: aSource.! !

!SdParserHelper methodsFor: 'interactive error handling' stamp: 'mt 4/11/2011 09:56:44.834'!
select
! !

!SdParserHelper methodsFor: 'interactive error handling' stamp: 'mt 4/11/2011 10:13:37.356'!
selectFrom: startIndex to: endIndex

	self selection: (startIndex to: endIndex).! !

!SdParserHelper methodsFor: 'accessing' stamp: 'mt 4/11/2011 10:13:18.346'!
selection

	^ selection! !

!SdParserHelper methodsFor: 'accessing' stamp: 'mt 4/11/2011 10:13:26.415'!
selection: anInterval

	selection := anInterval.! !

!SdParserHelper methodsFor: 'interactive error handling' stamp: 'mt 4/11/2011 09:40:44.679'!
selectionInterval

	^ 1 to: 0! !

!SdParserHelper methodsFor: 'accessing' stamp: 'mt 4/11/2011 09:41:19.665'!
text

	^ text! !

!SdParserHelper methodsFor: 'accessing' stamp: 'mt 4/11/2011 09:41:26.804'!
text: stringOrText

	text := stringOrText.! !

!SdParserHelper methodsFor: 'accessing' stamp: 'mt 4/11/2011 09:46:17.653'!
view

	^ view! !

!SdParserHelper methodsFor: 'accessing' stamp: 'mt 4/11/2011 09:46:25.661'!
view: aMorph

	view := aMorph.! !

UserInterface subclass: #SdCallTreePanelUi
	instanceVariableNames: 'testClassCmb testCmb callTree'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sight-Dynamic-UI'!

!SdCallTreePanelUi methodsFor: 'accessing' stamp: 'mt 3/30/2011 09:58:24.882'!
callTree
	"nil"
	
	^ callTree! !

!SdCallTreePanelUi methodsFor: 'ui - private' stamp: 'mt 3/30/2011 09:58:24.957'!
initializePanel
	"This code is auto-generated. DO NOT modify it because another generation step will overwrite all changes."
	
	panel := UiContainer new.
	panel changeTableLayout.
	panel extent: (200@472).
	panel position: (0@0).
	panel hResizing: (#rigid).
	panel vResizing: (#rigid).
	panel cellInset: (4).
	panel cellPositioning: (#center).
	panel cellSpacing: (#none).
	panel layoutInset: (4).
	panel listCentering: (#topLeft).
	panel listDirection: (#topToBottom).
	panel listSpacing: (#none).
	panel maxCellSize: (1073741823).
	panel minCellSize: (0).
	panel wrapCentering: (#topLeft).
	panel wrapDirection: (#none).
	panel frameFractions: (0@0 corner: 1@1).
	panel frameOffsets: (0@0 corner: 0@0).
	panel balloonText: (nil).
	panel balloonColor: ((TranslucentColor r: 0.91 g: 0.94 b: 1.0 alpha: 0.945)).
	panel color: (Color transparent).
	panel borderWidth: (0).
	panel borderStyle2: (#simple).
	panel borderColor: (Color black).
	panel setProperty: #objectName toValue: 'panel'.
	testClassCmb := UiComboBox new.
	testClassCmb changeTableLayout.
	testClassCmb extent: (192@21).
	testClassCmb position: (4@4).
	testClassCmb hResizing: (#spaceFill).
	testClassCmb vResizing: (#shrinkWrap).
	testClassCmb cellInset: (0).
	testClassCmb cellPositioning: (#topLeft).
	testClassCmb cellSpacing: (#none).
	testClassCmb layoutInset: (0).
	testClassCmb listCentering: (#topLeft).
	testClassCmb listDirection: (#leftToRight).
	testClassCmb listSpacing: (#none).
	testClassCmb maxCellSize: (1073741823).
	testClassCmb minCellSize: (0).
	testClassCmb wrapCentering: (#topLeft).
	testClassCmb wrapDirection: (#none).
	testClassCmb frameFractions: (0@0 corner: 1@1).
	testClassCmb frameOffsets: (0@0 corner: 0@0).
	testClassCmb balloonText: (nil).
	testClassCmb balloonColor: ((TranslucentColor r: 0.91 g: 0.94 b: 1.0 alpha: 0.949)).
	testClassCmb readOnly: (true).
	testClassCmb setProperty: #objectName toValue: 'testClassCmb'.
	panel addMorphBack: testClassCmb.
	testCmb := UiComboBox new.
	testCmb changeTableLayout.
	testCmb extent: (192@21).
	testCmb position: (4@29).
	testCmb hResizing: (#spaceFill).
	testCmb vResizing: (#shrinkWrap).
	testCmb cellInset: (0).
	testCmb cellPositioning: (#topLeft).
	testCmb cellSpacing: (#none).
	testCmb layoutInset: (0).
	testCmb listCentering: (#topLeft).
	testCmb listDirection: (#leftToRight).
	testCmb listSpacing: (#none).
	testCmb maxCellSize: (1073741823).
	testCmb minCellSize: (0).
	testCmb wrapCentering: (#topLeft).
	testCmb wrapDirection: (#none).
	testCmb frameFractions: (0@0 corner: 1@1).
	testCmb frameOffsets: (0@0 corner: 0@0).
	testCmb balloonText: (nil).
	testCmb balloonColor: ((TranslucentColor r: 0.91 g: 0.94 b: 1.0 alpha: 0.945)).
	testCmb readOnly: (true).
	testCmb setProperty: #objectName toValue: 'testCmb'.
	panel addMorphBack: testCmb.
	callTree := UiTreeView new.
	callTree extent: (192@414).
	callTree position: (4@54).
	callTree hResizing: (#spaceFill).
	callTree vResizing: (#spaceFill).
	callTree cellInset: (0).
	callTree cellPositioning: (#center).
	callTree cellSpacing: (#none).
	callTree layoutInset: (0).
	callTree listCentering: (#topLeft).
	callTree listDirection: (#topToBottom).
	callTree listSpacing: (#none).
	callTree maxCellSize: (1073741823).
	callTree minCellSize: (0).
	callTree wrapCentering: (#topLeft).
	callTree wrapDirection: (#none).
	callTree frameFractions: (0@0 corner: 1@1).
	callTree frameOffsets: (0@0 corner: 0@0).
	callTree modelClass: (nil).
	callTree horizontalHeaderVisible: (true).
	callTree verticalHeaderVisible: (false).
	callTree selectionBehavior: (#selectRows).
	callTree selectionColor: ((Color r: 0.772 g: 0.858 b: 1.0)).
	callTree selectionMode: (UiViewSingleSelection).
	callTree setProperty: #objectName toValue: 'callTree'.
	panel addMorphBack: callTree.
	panel setProperty: #uiClassName toValue: 'SdCallTreePanel'.! !

!SdCallTreePanelUi methodsFor: 'accessing' stamp: 'mt 3/30/2011 09:58:24.756'!
testClassCmb
	"nil"
	
	^ testClassCmb! !

!SdCallTreePanelUi methodsFor: 'accessing' stamp: 'mt 3/30/2011 09:58:24.836'!
testCmb
	"nil"
	
	^ testCmb! !

UserInterface subclass: #SdClassEditorUi
	instanceVariableNames: 'unnamedMorph1 unnamedMorph2 unnamedMorph3 classLbl unnamedMorph4 closeBtn topContainer unnamedMorph5 findBtn commentBtn definitionBtn unnamedMorph6 saveAllBtn closeAllBtn browseBtn codeArea'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sight-Dynamic-UI'!

!SdClassEditorUi methodsFor: 'accessing' stamp: 'mt 4/13/2011 15:41:41.088'!
browseBtn
	"nil"
	
	^ browseBtn! !

!SdClassEditorUi methodsFor: 'accessing' stamp: 'mt 4/13/2011 15:41:40.602'!
classLbl
	"nil"
	
	^ classLbl! !

!SdClassEditorUi methodsFor: 'accessing' stamp: 'mt 4/13/2011 15:41:41.038'!
closeAllBtn
	"nil"
	
	^ closeAllBtn! !

!SdClassEditorUi methodsFor: 'accessing' stamp: 'mt 4/13/2011 15:41:40.678'!
closeBtn
	"nil"
	
	^ closeBtn! !

!SdClassEditorUi methodsFor: 'accessing' stamp: 'mt 4/13/2011 15:41:41.155'!
codeArea
	"nil"
	
	^ codeArea! !

!SdClassEditorUi methodsFor: 'accessing' stamp: 'mt 4/13/2011 15:41:40.851'!
commentBtn
	"nil"
	
	^ commentBtn! !

!SdClassEditorUi methodsFor: 'accessing' stamp: 'mt 4/13/2011 15:41:40.921'!
definitionBtn
	"nil"
	
	^ definitionBtn! !

!SdClassEditorUi methodsFor: 'accessing' stamp: 'mt 4/13/2011 15:41:40.793'!
findBtn
	"nil"
	
	^ findBtn! !

!SdClassEditorUi methodsFor: 'ui - private' stamp: 'mt 4/13/2011 15:41:41.254'!
initializePanel
	"This code is auto-generated. DO NOT modify it because another generation step will overwrite all changes."
	
	panel := UiContainer new.
	panel changeTableLayout.
	panel extent: (244@569).
	panel position: (0@0).
	panel hResizing: (#rigid).
	panel vResizing: (#rigid).
	panel cellInset: (2).
	panel cellPositioning: (#center).
	panel cellSpacing: (#none).
	panel layoutInset: (4).
	panel listCentering: (#topLeft).
	panel listDirection: (#topToBottom).
	panel listSpacing: (#none).
	panel maxCellSize: (1073741823).
	panel minCellSize: (0).
	panel wrapCentering: (#topLeft).
	panel wrapDirection: (#none).
	panel frameFractions: (0@0 corner: 1@1).
	panel frameOffsets: (0@0 corner: 0@0).
	panel balloonText: (nil).
	panel balloonColor: ((TranslucentColor r: 0.91 g: 0.94 b: 1.0 alpha: 0.922)).
	panel color: (Color transparent).
	panel borderWidth: (0).
	panel borderStyle2: (#simple).
	panel borderColor: (Color black).
	panel setProperty: #objectName toValue: 'panel'.
	unnamedMorph1 := UiContainer new.
	unnamedMorph1 changeTableLayout.
	unnamedMorph1 extent: (236@19).
	unnamedMorph1 position: (4@4).
	unnamedMorph1 hResizing: (#spaceFill).
	unnamedMorph1 vResizing: (#shrinkWrap).
	unnamedMorph1 cellInset: (4).
	unnamedMorph1 cellPositioning: (#center).
	unnamedMorph1 cellSpacing: (#none).
	unnamedMorph1 layoutInset: (0).
	unnamedMorph1 listCentering: (#center).
	unnamedMorph1 listDirection: (#leftToRight).
	unnamedMorph1 listSpacing: (#none).
	unnamedMorph1 maxCellSize: (1073741823).
	unnamedMorph1 minCellSize: (0).
	unnamedMorph1 wrapCentering: (#topLeft).
	unnamedMorph1 wrapDirection: (#none).
	unnamedMorph1 frameFractions: (0@0 corner: 1@1).
	unnamedMorph1 frameOffsets: (0@0 corner: 0@0).
	unnamedMorph1 balloonText: (nil).
	unnamedMorph1 balloonColor: ((TranslucentColor r: 0.91 g: 0.94 b: 1.0 alpha: 0.929)).
	unnamedMorph1 color: (Color transparent).
	unnamedMorph1 borderWidth: (0).
	unnamedMorph1 borderStyle2: (#simple).
	unnamedMorph1 borderColor: (Color black).
	unnamedMorph2 := UiContainer new.
	unnamedMorph2 extent: (20@5).
	unnamedMorph2 position: (7@11).
	unnamedMorph2 hResizing: (#rigid).
	unnamedMorph2 vResizing: (#rigid).
	unnamedMorph2 cellInset: (4).
	unnamedMorph2 cellPositioning: (#center).
	unnamedMorph2 cellSpacing: (#none).
	unnamedMorph2 layoutInset: (4).
	unnamedMorph2 listCentering: (#topLeft).
	unnamedMorph2 listDirection: (#topToBottom).
	unnamedMorph2 listSpacing: (#none).
	unnamedMorph2 maxCellSize: (1073741823).
	unnamedMorph2 minCellSize: (0).
	unnamedMorph2 wrapCentering: (#topLeft).
	unnamedMorph2 wrapDirection: (#none).
	unnamedMorph2 frameFractions: (0@0 corner: 1@1).
	unnamedMorph2 frameOffsets: (0@0 corner: 0@0).
	unnamedMorph2 balloonText: (nil).
	unnamedMorph2 balloonColor: ((TranslucentColor r: 0.91 g: 0.94 b: 1.0 alpha: 0.929)).
	unnamedMorph2 color: (Color transparent).
	unnamedMorph2 borderWidth: (0).
	unnamedMorph2 borderStyle2: (#simple).
	unnamedMorph2 borderColor: (Color black).
	unnamedMorph1 addMorphBack: unnamedMorph2.
	unnamedMorph3 := UiSpacer new.
	unnamedMorph3 extent: (72@10).
	unnamedMorph3 position: (34@8).
	unnamedMorph3 hResizing: (#spaceFill).
	unnamedMorph3 vResizing: (#rigid).
	unnamedMorph3 cellInset: (4).
	unnamedMorph3 cellPositioning: (#center).
	unnamedMorph3 cellSpacing: (#none).
	unnamedMorph3 layoutInset: (4).
	unnamedMorph3 listCentering: (#topLeft).
	unnamedMorph3 listDirection: (#topToBottom).
	unnamedMorph3 listSpacing: (#none).
	unnamedMorph3 maxCellSize: (1073741823).
	unnamedMorph3 minCellSize: (0).
	unnamedMorph3 wrapCentering: (#topLeft).
	unnamedMorph3 wrapDirection: (#none).
	unnamedMorph3 frameFractions: (0@0 corner: 1@1).
	unnamedMorph3 frameOffsets: (0@0 corner: 0@0).
	unnamedMorph3 balloonText: (nil).
	unnamedMorph3 balloonColor: ((TranslucentColor r: 0.91 g: 0.94 b: 1.0 alpha: 0.929)).
	unnamedMorph3 color: (Color blue).
	unnamedMorph3 borderWidth: (0).
	unnamedMorph3 borderStyle2: (#simple).
	unnamedMorph3 borderColor: (Color black).
	unnamedMorph3 orientation: (#horizontal).
	unnamedMorph1 addMorphBack: unnamedMorph3.
	classLbl := UiLabel new.
	classLbl changeTableLayout.
	classLbl extent: (27@19).
	classLbl position: (110@4).
	classLbl hResizing: (#shrinkWrap).
	classLbl vResizing: (#shrinkWrap).
	classLbl cellInset: (0).
	classLbl cellPositioning: (#center).
	classLbl cellSpacing: (#none).
	classLbl layoutInset: (0).
	classLbl listCentering: (#topLeft).
	classLbl listDirection: (#leftToRight).
	classLbl listSpacing: (#none).
	classLbl maxCellSize: (1073741823).
	classLbl minCellSize: (0).
	classLbl wrapCentering: (#topLeft).
	classLbl wrapDirection: (#none).
	classLbl frameFractions: (0@0 corner: 1@1).
	classLbl frameOffsets: (0@0 corner: 0@0).
	classLbl balloonText: (nil).
	classLbl balloonColor: ((TranslucentColor r: 0.91 g: 0.94 b: 1.0 alpha: 0.922)).
	classLbl text: ('Class').
	classLbl setProperty: #formByDesigner toValue: 'nil'.
	classLbl form: (nil).
	classLbl setProperty: #objectName toValue: 'classLbl'.
	unnamedMorph1 addMorphBack: classLbl.
	unnamedMorph4 := UiSpacer new.
	unnamedMorph4 extent: (73@12).
	unnamedMorph4 position: (141@7).
	unnamedMorph4 hResizing: (#spaceFill).
	unnamedMorph4 vResizing: (#rigid).
	unnamedMorph4 cellInset: (4).
	unnamedMorph4 cellPositioning: (#center).
	unnamedMorph4 cellSpacing: (#none).
	unnamedMorph4 layoutInset: (4).
	unnamedMorph4 listCentering: (#topLeft).
	unnamedMorph4 listDirection: (#topToBottom).
	unnamedMorph4 listSpacing: (#none).
	unnamedMorph4 maxCellSize: (1073741823).
	unnamedMorph4 minCellSize: (0).
	unnamedMorph4 wrapCentering: (#topLeft).
	unnamedMorph4 wrapDirection: (#none).
	unnamedMorph4 frameFractions: (0@0 corner: 1@1).
	unnamedMorph4 frameOffsets: (0@0 corner: 0@0).
	unnamedMorph4 balloonText: (nil).
	unnamedMorph4 balloonColor: ((TranslucentColor r: 0.91 g: 0.94 b: 1.0 alpha: 0.929)).
	unnamedMorph4 color: (Color blue).
	unnamedMorph4 borderWidth: (0).
	unnamedMorph4 borderStyle2: (#simple).
	unnamedMorph4 borderColor: (Color black).
	unnamedMorph4 orientation: (#horizontal).
	unnamedMorph1 addMorphBack: unnamedMorph4.
	closeBtn := UiToolButton new.
	closeBtn changeTableLayout.
	closeBtn extent: (22@18).
	closeBtn position: (218@4).
	closeBtn hResizing: (#shrinkWrap).
	closeBtn vResizing: (#shrinkWrap).
	closeBtn cellInset: (4).
	closeBtn cellPositioning: (#center).
	closeBtn cellSpacing: (#none).
	closeBtn layoutInset: (2@0 corner: 2@0).
	closeBtn listCentering: (#topLeft).
	closeBtn listDirection: (#leftToRight).
	closeBtn listSpacing: (#none).
	closeBtn maxCellSize: (1073741823).
	closeBtn minCellSize: (0).
	closeBtn wrapCentering: (#topLeft).
	closeBtn wrapDirection: (#none).
	closeBtn frameFractions: (0@0 corner: 1@1).
	closeBtn frameOffsets: (0@0 corner: 0@0).
	closeBtn balloonText: (nil).
	closeBtn balloonColor: ((TranslucentColor r: 0.91 g: 0.94 b: 1.0 alpha: 0.929)).
	closeBtn enabled: (true).
	closeBtn setProperty: #iconByDesigner toValue: 'UiSilkIcons deleteIcon'.
	closeBtn icon: (UiSilkIcons deleteIcon).
	closeBtn text: ('ToolButton').
	closeBtn color: ((Color r: 0.85 g: 0.85 b: 0.85)).
	closeBtn checkable: (false).
	closeBtn checked: (false).
	closeBtn autoExclusive: (false).
	closeBtn autoRaise: (true).
	closeBtn buttonStyle: (#iconOnly).
	closeBtn setProperty: #objectName toValue: 'closeBtn'.
	unnamedMorph1 addMorphBack: closeBtn.
	panel addMorphBack: unnamedMorph1.
	topContainer := UiContainer new.
	topContainer changeTableLayout.
	topContainer extent: (236@21).
	topContainer position: (4@24).
	topContainer hResizing: (#spaceFill).
	topContainer vResizing: (#shrinkWrap).
	topContainer cellInset: (4).
	topContainer cellPositioning: (#center).
	topContainer cellSpacing: (#none).
	topContainer layoutInset: (0).
	topContainer listCentering: (#topLeft).
	topContainer listDirection: (#topToBottom).
	topContainer listSpacing: (#none).
	topContainer maxCellSize: (1073741823).
	topContainer minCellSize: (0).
	topContainer wrapCentering: (#topLeft).
	topContainer wrapDirection: (#none).
	topContainer frameFractions: (0@0 corner: 1@1).
	topContainer frameOffsets: (0@0 corner: 0@0).
	topContainer balloonText: (nil).
	topContainer balloonColor: ((TranslucentColor r: 0.91 g: 0.94 b: 1.0 alpha: 0.922)).
	topContainer color: (Color transparent).
	topContainer borderWidth: (0).
	topContainer borderStyle2: (#simple).
	topContainer borderColor: (Color black).
	topContainer setProperty: #objectName toValue: 'topContainer'.
	unnamedMorph5 := UiContainer new.
	unnamedMorph5 changeTableLayout.
	unnamedMorph5 extent: (236@18).
	unnamedMorph5 position: (4@24).
	unnamedMorph5 hResizing: (#spaceFill).
	unnamedMorph5 vResizing: (#shrinkWrap).
	unnamedMorph5 cellInset: (0).
	unnamedMorph5 cellPositioning: (#center).
	unnamedMorph5 cellSpacing: (#none).
	unnamedMorph5 layoutInset: (0).
	unnamedMorph5 listCentering: (#topLeft).
	unnamedMorph5 listDirection: (#leftToRight).
	unnamedMorph5 listSpacing: (#none).
	unnamedMorph5 maxCellSize: (1073741823).
	unnamedMorph5 minCellSize: (0).
	unnamedMorph5 wrapCentering: (#topLeft).
	unnamedMorph5 wrapDirection: (#none).
	unnamedMorph5 frameFractions: (0@0 corner: 1@1).
	unnamedMorph5 frameOffsets: (0@0 corner: 0@0).
	unnamedMorph5 balloonText: (nil).
	unnamedMorph5 balloonColor: ((TranslucentColor r: 0.91 g: 0.94 b: 1.0 alpha: 0.922)).
	unnamedMorph5 color: (Color transparent).
	unnamedMorph5 borderWidth: (0).
	unnamedMorph5 borderStyle2: (#simple).
	unnamedMorph5 borderColor: (Color black).
	findBtn := UiToolButton new.
	findBtn changeTableLayout.
	findBtn extent: (22@18).
	findBtn position: (4@24).
	findBtn hResizing: (#shrinkWrap).
	findBtn vResizing: (#shrinkWrap).
	findBtn cellInset: (4).
	findBtn cellPositioning: (#center).
	findBtn cellSpacing: (#none).
	findBtn layoutInset: (2@0 corner: 2@0).
	findBtn listCentering: (#topLeft).
	findBtn listDirection: (#leftToRight).
	findBtn listSpacing: (#none).
	findBtn maxCellSize: (1073741823).
	findBtn minCellSize: (0).
	findBtn wrapCentering: (#topLeft).
	findBtn wrapDirection: (#none).
	findBtn frameFractions: (0@0 corner: 1@1).
	findBtn frameOffsets: (0@0 corner: 0@0).
	findBtn balloonText: ('Find Method').
	findBtn balloonColor: ((TranslucentColor r: 0.91 g: 0.94 b: 1.0 alpha: 0.922)).
	findBtn enabled: (true).
	findBtn setProperty: #iconByDesigner toValue: 'UiFugueIcons magnifierIcon'.
	findBtn icon: (UiFugueIcons magnifierIcon).
	findBtn text: ('ToolButton').
	findBtn color: ((Color r: 0.85 g: 0.85 b: 0.85)).
	findBtn checkable: (true).
	findBtn checked: (false).
	findBtn autoExclusive: (false).
	findBtn autoRaise: (true).
	findBtn buttonStyle: (#iconOnly).
	findBtn setProperty: #objectName toValue: 'findBtn'.
	unnamedMorph5 addMorphBack: findBtn.
	commentBtn := UiToolButton new.
	commentBtn changeTableLayout.
	commentBtn extent: (22@18).
	commentBtn position: (26@24).
	commentBtn hResizing: (#shrinkWrap).
	commentBtn vResizing: (#shrinkWrap).
	commentBtn cellInset: (4).
	commentBtn cellPositioning: (#center).
	commentBtn cellSpacing: (#none).
	commentBtn layoutInset: (2@0 corner: 2@0).
	commentBtn listCentering: (#topLeft).
	commentBtn listDirection: (#leftToRight).
	commentBtn listSpacing: (#none).
	commentBtn maxCellSize: (1073741823).
	commentBtn minCellSize: (0).
	commentBtn wrapCentering: (#topLeft).
	commentBtn wrapDirection: (#none).
	commentBtn frameFractions: (0@0 corner: 1@1).
	commentBtn frameOffsets: (0@0 corner: 0@0).
	commentBtn balloonText: ('Edit class comment').
	commentBtn balloonColor: ((TranslucentColor r: 0.91 g: 0.94 b: 1.0 alpha: 0.922)).
	commentBtn enabled: (true).
	commentBtn setProperty: #iconByDesigner toValue: 'UiSilkIcons commentIcon'.
	commentBtn icon: (UiSilkIcons commentIcon).
	commentBtn text: ('ToolButton').
	commentBtn color: ((Color r: 0.85 g: 0.85 b: 0.85)).
	commentBtn checkable: (true).
	commentBtn checked: (false).
	commentBtn autoExclusive: (false).
	commentBtn autoRaise: (true).
	commentBtn buttonStyle: (#iconOnly).
	commentBtn setProperty: #objectName toValue: 'commentBtn'.
	unnamedMorph5 addMorphBack: commentBtn.
	definitionBtn := UiToolButton new.
	definitionBtn changeTableLayout.
	definitionBtn extent: (22@18).
	definitionBtn position: (48@24).
	definitionBtn hResizing: (#shrinkWrap).
	definitionBtn vResizing: (#shrinkWrap).
	definitionBtn cellInset: (4).
	definitionBtn cellPositioning: (#center).
	definitionBtn cellSpacing: (#none).
	definitionBtn layoutInset: (2@0 corner: 2@0).
	definitionBtn listCentering: (#topLeft).
	definitionBtn listDirection: (#leftToRight).
	definitionBtn listSpacing: (#none).
	definitionBtn maxCellSize: (1073741823).
	definitionBtn minCellSize: (0).
	definitionBtn wrapCentering: (#topLeft).
	definitionBtn wrapDirection: (#none).
	definitionBtn frameFractions: (0@0 corner: 1@1).
	definitionBtn frameOffsets: (0@0 corner: 0@0).
	definitionBtn balloonText: (nil).
	definitionBtn balloonColor: ((TranslucentColor r: 0.91 g: 0.94 b: 1.0 alpha: 0.941)).
	definitionBtn enabled: (true).
	definitionBtn setProperty: #iconByDesigner toValue: 'UiSilkIcons wrenchOrangeIcon'.
	definitionBtn icon: (UiSilkIcons wrenchOrangeIcon).
	definitionBtn text: ('ToolButton').
	definitionBtn color: ((Color r: 0.85 g: 0.85 b: 0.85)).
	definitionBtn checkable: (true).
	definitionBtn checked: (false).
	definitionBtn autoExclusive: (false).
	definitionBtn autoRaise: (true).
	definitionBtn buttonStyle: (#iconOnly).
	definitionBtn setProperty: #objectName toValue: 'definitionBtn'.
	unnamedMorph5 addMorphBack: definitionBtn.
	unnamedMorph6 := UiSpacer new.
	unnamedMorph6 extent: (104@13).
	unnamedMorph6 position: (70@27).
	unnamedMorph6 hResizing: (#spaceFill).
	unnamedMorph6 vResizing: (#rigid).
	unnamedMorph6 cellInset: (4).
	unnamedMorph6 cellPositioning: (#center).
	unnamedMorph6 cellSpacing: (#none).
	unnamedMorph6 layoutInset: (4).
	unnamedMorph6 listCentering: (#topLeft).
	unnamedMorph6 listDirection: (#topToBottom).
	unnamedMorph6 listSpacing: (#none).
	unnamedMorph6 maxCellSize: (1073741823).
	unnamedMorph6 minCellSize: (0).
	unnamedMorph6 wrapCentering: (#topLeft).
	unnamedMorph6 wrapDirection: (#none).
	unnamedMorph6 frameFractions: (0@0 corner: 1@1).
	unnamedMorph6 frameOffsets: (0@0 corner: 0@0).
	unnamedMorph6 balloonText: (nil).
	unnamedMorph6 balloonColor: ((TranslucentColor r: 0.91 g: 0.94 b: 1.0 alpha: 0.922)).
	unnamedMorph6 color: (Color blue).
	unnamedMorph6 borderWidth: (0).
	unnamedMorph6 borderStyle2: (#simple).
	unnamedMorph6 borderColor: (Color black).
	unnamedMorph6 orientation: (#horizontal).
	unnamedMorph5 addMorphBack: unnamedMorph6.
	saveAllBtn := UiToolButton new.
	saveAllBtn changeTableLayout.
	saveAllBtn extent: (22@18).
	saveAllBtn position: (174@24).
	saveAllBtn hResizing: (#shrinkWrap).
	saveAllBtn vResizing: (#shrinkWrap).
	saveAllBtn cellInset: (4).
	saveAllBtn cellPositioning: (#center).
	saveAllBtn cellSpacing: (#none).
	saveAllBtn layoutInset: (2@0 corner: 2@0).
	saveAllBtn listCentering: (#topLeft).
	saveAllBtn listDirection: (#leftToRight).
	saveAllBtn listSpacing: (#none).
	saveAllBtn maxCellSize: (1073741823).
	saveAllBtn minCellSize: (0).
	saveAllBtn wrapCentering: (#topLeft).
	saveAllBtn wrapDirection: (#none).
	saveAllBtn frameFractions: (0@0 corner: 1@1).
	saveAllBtn frameOffsets: (0@0 corner: 0@0).
	saveAllBtn balloonText: ('Save all changed methods').
	saveAllBtn balloonColor: ((TranslucentColor r: 0.91 g: 0.94 b: 1.0 alpha: 0.937)).
	saveAllBtn enabled: (true).
	saveAllBtn setProperty: #iconByDesigner toValue: 'UiSilkIcons diskMultipleIcon'.
	saveAllBtn icon: (UiSilkIcons diskMultipleIcon).
	saveAllBtn text: ('ToolButton').
	saveAllBtn color: ((Color r: 0.85 g: 0.85 b: 0.85)).
	saveAllBtn checkable: (false).
	saveAllBtn checked: (false).
	saveAllBtn autoExclusive: (false).
	saveAllBtn autoRaise: (true).
	saveAllBtn buttonStyle: (#iconOnly).
	saveAllBtn setProperty: #objectName toValue: 'saveAllBtn'.
	unnamedMorph5 addMorphBack: saveAllBtn.
	closeAllBtn := UiToolButton new.
	closeAllBtn changeTableLayout.
	closeAllBtn extent: (22@18).
	closeAllBtn position: (196@24).
	closeAllBtn hResizing: (#shrinkWrap).
	closeAllBtn vResizing: (#shrinkWrap).
	closeAllBtn cellInset: (4).
	closeAllBtn cellPositioning: (#center).
	closeAllBtn cellSpacing: (#none).
	closeAllBtn layoutInset: (2@0 corner: 2@0).
	closeAllBtn listCentering: (#topLeft).
	closeAllBtn listDirection: (#leftToRight).
	closeAllBtn listSpacing: (#none).
	closeAllBtn maxCellSize: (1073741823).
	closeAllBtn minCellSize: (0).
	closeAllBtn wrapCentering: (#topLeft).
	closeAllBtn wrapDirection: (#none).
	closeAllBtn frameFractions: (0@0 corner: 1@1).
	closeAllBtn frameOffsets: (0@0 corner: 0@0).
	closeAllBtn balloonText: ('Close all editors for this
class').
	closeAllBtn balloonColor: ((TranslucentColor r: 0.91 g: 0.94 b: 1.0 alpha: 0.933)).
	closeAllBtn enabled: (true).
	closeAllBtn setProperty: #iconByDesigner toValue: 'UiFugueIcons broomIcon'.
	closeAllBtn icon: (UiFugueIcons broomIcon).
	closeAllBtn text: ('ToolButton').
	closeAllBtn color: ((Color r: 0.85 g: 0.85 b: 0.85)).
	closeAllBtn checkable: (false).
	closeAllBtn checked: (false).
	closeAllBtn autoExclusive: (false).
	closeAllBtn autoRaise: (true).
	closeAllBtn buttonStyle: (#iconOnly).
	closeAllBtn setProperty: #objectName toValue: 'closeAllBtn'.
	unnamedMorph5 addMorphBack: closeAllBtn.
	browseBtn := UiToolButton new.
	browseBtn changeTableLayout.
	browseBtn extent: (22@18).
	browseBtn position: (218@24).
	browseBtn hResizing: (#shrinkWrap).
	browseBtn vResizing: (#shrinkWrap).
	browseBtn cellInset: (4).
	browseBtn cellPositioning: (#center).
	browseBtn cellSpacing: (#none).
	browseBtn layoutInset: (2@0 corner: 2@0).
	browseBtn listCentering: (#topLeft).
	browseBtn listDirection: (#leftToRight).
	browseBtn listSpacing: (#none).
	browseBtn maxCellSize: (1073741823).
	browseBtn minCellSize: (0).
	browseBtn wrapCentering: (#topLeft).
	browseBtn wrapDirection: (#none).
	browseBtn frameFractions: (0@0 corner: 1@1).
	browseBtn frameOffsets: (0@0 corner: 0@0).
	browseBtn balloonText: ('Browse Classes').
	browseBtn balloonColor: ((TranslucentColor r: 0.91 g: 0.94 b: 1.0 alpha: 0.922)).
	browseBtn enabled: (true).
	browseBtn setProperty: #iconByDesigner toValue: 'UiSilkIcons folderExploreIcon'.
	browseBtn icon: (UiSilkIcons folderExploreIcon).
	browseBtn text: ('ToolButton').
	browseBtn color: ((Color r: 0.85 g: 0.85 b: 0.85)).
	browseBtn checkable: (false).
	browseBtn checked: (false).
	browseBtn autoExclusive: (false).
	browseBtn autoRaise: (true).
	browseBtn buttonStyle: (#iconOnly).
	browseBtn setProperty: #objectName toValue: 'browseBtn'.
	unnamedMorph5 addMorphBack: browseBtn.
	topContainer addMorphBack: unnamedMorph5.
	panel addMorphBack: topContainer.
	codeArea := UiScrollArea new.
	codeArea extent: (236@520).
	codeArea position: (4@45).
	codeArea hResizing: (#spaceFill).
	codeArea vResizing: (#spaceFill).
	codeArea cellInset: (0).
	codeArea cellPositioning: (#center).
	codeArea cellSpacing: (#none).
	codeArea layoutInset: (0).
	codeArea listCentering: (#topLeft).
	codeArea listDirection: (#topToBottom).
	codeArea listSpacing: (#none).
	codeArea maxCellSize: (1073741823).
	codeArea minCellSize: (0).
	codeArea wrapCentering: (#topLeft).
	codeArea wrapDirection: (#none).
	codeArea frameFractions: (0@0 corner: 1@1).
	codeArea frameOffsets: (0@0 corner: 0@0).
	codeArea balloonText: (nil).
	codeArea balloonColor: ((TranslucentColor r: 0.91 g: 0.94 b: 1.0 alpha: 0.922)).
	codeArea widgetResizable: (true).
	codeArea horizontalScrollBarPolicy: (#alwaysOff).
	codeArea verticalScrollBarPolicy: (#asNeeded).
	codeArea outerBorderWidth: (0).
	codeArea setProperty: #objectName toValue: 'codeArea'.
	panel addMorphBack: codeArea.
	panel setProperty: #uiClassName toValue: 'SdClassEditor'.! !

!SdClassEditorUi methodsFor: 'accessing' stamp: 'mt 4/13/2011 15:41:40.963'!
saveAllBtn
	"nil"
	
	^ saveAllBtn! !

!SdClassEditorUi methodsFor: 'accessing' stamp: 'mt 4/13/2011 15:41:40.739'!
topContainer
	"nil"
	
	^ topContainer! !

UserInterface subclass: #SdClassSelectorUi
	instanceVariableNames: 'unnamedMorph1 container unnamedMorph2 recentlyList'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sight-Dynamic-UI'!

!SdClassSelectorUi methodsFor: 'accessing' stamp: 'mt 4/17/2011 12:12:25.765'!
container
	"nil"
	
	^ container! !

!SdClassSelectorUi methodsFor: 'ui - private' stamp: 'mt 4/17/2011 12:12:25.864'!
initializePanel
	"This code is auto-generated. DO NOT modify it because another generation step will overwrite all changes."
	
	panel := UiContainer new.
	panel changeTableLayout.
	panel extent: (248@388).
	panel position: (0@0).
	panel hResizing: (#rigid).
	panel vResizing: (#rigid).
	panel cellInset: (4).
	panel cellPositioning: (#center).
	panel cellSpacing: (#none).
	panel layoutInset: (4).
	panel listCentering: (#topLeft).
	panel listDirection: (#topToBottom).
	panel listSpacing: (#none).
	panel maxCellSize: (1073741823).
	panel minCellSize: (0).
	panel wrapCentering: (#topLeft).
	panel wrapDirection: (#none).
	panel frameFractions: (0@0 corner: 1@1).
	panel frameOffsets: (0@0 corner: 0@0).
	panel balloonText: ('').
	panel balloonColor: ((TranslucentColor r: 0.91 g: 0.94 b: 1.0 alpha: 0.941)).
	panel color: (Color transparent).
	panel borderWidth: (0).
	panel borderStyle2: (#simple).
	panel borderColor: (Color black).
	panel setProperty: #objectName toValue: 'panel'.
	unnamedMorph1 := UiLabel new.
	unnamedMorph1 changeTableLayout.
	unnamedMorph1 extent: (78@19).
	unnamedMorph1 position: (85@4).
	unnamedMorph1 hResizing: (#shrinkWrap).
	unnamedMorph1 vResizing: (#shrinkWrap).
	unnamedMorph1 cellInset: (0).
	unnamedMorph1 cellPositioning: (#center).
	unnamedMorph1 cellSpacing: (#none).
	unnamedMorph1 layoutInset: (0).
	unnamedMorph1 listCentering: (#topLeft).
	unnamedMorph1 listDirection: (#leftToRight).
	unnamedMorph1 listSpacing: (#none).
	unnamedMorph1 maxCellSize: (1073741823).
	unnamedMorph1 minCellSize: (0).
	unnamedMorph1 wrapCentering: (#topLeft).
	unnamedMorph1 wrapDirection: (#none).
	unnamedMorph1 frameFractions: (0@0 corner: 1@1).
	unnamedMorph1 frameOffsets: (0@0 corner: 0@0).
	unnamedMorph1 balloonText: (nil).
	unnamedMorph1 balloonColor: ((TranslucentColor r: 0.91 g: 0.94 b: 1.0 alpha: 0.937)).
	unnamedMorph1 text: ('Choose a Class').
	unnamedMorph1 setProperty: #formByDesigner toValue: 'nil'.
	unnamedMorph1 form: (nil).
	panel addMorphBack: unnamedMorph1.
	container := UiContainer new.
	container changeTableLayout.
	container extent: (240@18).
	container position: (4@30).
	container hResizing: (#spaceFill).
	container vResizing: (#shrinkWrap).
	container cellInset: (0).
	container cellPositioning: (#center).
	container cellSpacing: (#none).
	container layoutInset: (0).
	container listCentering: (#topLeft).
	container listDirection: (#topToBottom).
	container listSpacing: (#none).
	container maxCellSize: (1073741823).
	container minCellSize: (0).
	container wrapCentering: (#topLeft).
	container wrapDirection: (#none).
	container frameFractions: (0@0 corner: 1@1).
	container frameOffsets: (0@0 corner: 0@0).
	container balloonText: (nil).
	container balloonColor: ((TranslucentColor r: 0.91 g: 0.94 b: 1.0 alpha: 0.941)).
	container color: (Color transparent).
	container borderWidth: (0).
	container borderStyle2: (#simple).
	container borderColor: (Color black).
	container setProperty: #objectName toValue: 'container'.
	panel addMorphBack: container.
	unnamedMorph2 := UiLabel new.
	unnamedMorph2 changeTableLayout.
	unnamedMorph2 extent: (80@19).
	unnamedMorph2 position: (84@55).
	unnamedMorph2 hResizing: (#shrinkWrap).
	unnamedMorph2 vResizing: (#shrinkWrap).
	unnamedMorph2 cellInset: (0).
	unnamedMorph2 cellPositioning: (#center).
	unnamedMorph2 cellSpacing: (#none).
	unnamedMorph2 layoutInset: (0).
	unnamedMorph2 listCentering: (#topLeft).
	unnamedMorph2 listDirection: (#leftToRight).
	unnamedMorph2 listSpacing: (#none).
	unnamedMorph2 maxCellSize: (1073741823).
	unnamedMorph2 minCellSize: (0).
	unnamedMorph2 wrapCentering: (#topLeft).
	unnamedMorph2 wrapDirection: (#none).
	unnamedMorph2 frameFractions: (0@0 corner: 1@1).
	unnamedMorph2 frameOffsets: (0@0 corner: 0@0).
	unnamedMorph2 balloonText: ('').
	unnamedMorph2 balloonColor: ((TranslucentColor r: 0.91 g: 0.94 b: 1.0 alpha: 0.922)).
	unnamedMorph2 text: ('Recently Used').
	unnamedMorph2 setProperty: #formByDesigner toValue: 'nil'.
	unnamedMorph2 form: (nil).
	panel addMorphBack: unnamedMorph2.
	recentlyList := UiListWidget new.
	recentlyList extent: (240@306).
	recentlyList position: (4@78).
	recentlyList hResizing: (#spaceFill).
	recentlyList vResizing: (#spaceFill).
	recentlyList cellInset: (0).
	recentlyList cellPositioning: (#center).
	recentlyList cellSpacing: (#none).
	recentlyList layoutInset: (0).
	recentlyList listCentering: (#topLeft).
	recentlyList listDirection: (#topToBottom).
	recentlyList listSpacing: (#none).
	recentlyList maxCellSize: (1073741823).
	recentlyList minCellSize: (0).
	recentlyList wrapCentering: (#topLeft).
	recentlyList wrapDirection: (#none).
	recentlyList frameFractions: (0@0 corner: 1@1).
	recentlyList frameOffsets: (0@0 corner: 0@0).
	recentlyList balloonText: (nil).
	recentlyList balloonColor: ((TranslucentColor r: 0.91 g: 0.94 b: 1.0 alpha: 0.945)).
	recentlyList widgetResizable: (false).
	recentlyList horizontalScrollBarPolicy: (#alwaysOff).
	recentlyList verticalScrollBarPolicy: (#asNeeded).
	recentlyList outerBorderWidth: (0).
	recentlyList modelClass: (UiListWidgetModel).
	recentlyList horizontalHeaderVisible: (false).
	recentlyList verticalHeaderVisible: (false).
	recentlyList selectionBehavior: (#selectRows).
	recentlyList selectionColor: ((Color r: 0.772 g: 0.858 b: 1.0)).
	recentlyList selectionMode: (UiViewSingleSelection).
	recentlyList setProperty: #objectName toValue: 'recentlyList'.
	panel addMorphBack: recentlyList.
	panel setProperty: #uiClassName toValue: 'SdClassSelector'.! !

!SdClassSelectorUi methodsFor: 'accessing' stamp: 'mt 4/17/2011 12:12:25.811'!
recentlyList
	"nil"
	
	^ recentlyList! !

UserInterface subclass: #SdMethodFinderUi
	instanceVariableNames: 'inputContainer protocolBtn searchField actionBtn resultList'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sight-Dynamic-UI'!

!SdMethodFinderUi methodsFor: 'accessing' stamp: 'mt 4/14/2011 11:25:09.322'!
actionBtn
	"nil"
	
	^ actionBtn! !

!SdMethodFinderUi methodsFor: 'ui - private' stamp: 'mt 4/14/2011 11:25:09.486'!
initializePanel
	"This code is auto-generated. DO NOT modify it because another generation step will overwrite all changes."
	
	panel := UiContainer new.
	panel changeTableLayout.
	panel extent: (218@100).
	panel position: (0@0).
	panel hResizing: (#rigid).
	panel vResizing: (#shrinkWrap).
	panel cellInset: (0).
	panel cellPositioning: (#center).
	panel cellSpacing: (#none).
	panel layoutInset: (4).
	panel listCentering: (#topLeft).
	panel listDirection: (#topToBottom).
	panel listSpacing: (#none).
	panel maxCellSize: (1073741823).
	panel minCellSize: (0).
	panel wrapCentering: (#topLeft).
	panel wrapDirection: (#none).
	panel frameFractions: (0@0 corner: 1@1).
	panel frameOffsets: (0@0 corner: 0@0).
	panel balloonText: (nil).
	panel balloonColor: ((TranslucentColor r: 0.91 g: 0.94 b: 1.0 alpha: 0.922)).
	panel color: (Color transparent).
	panel borderWidth: (0).
	panel borderStyle2: (#simple).
	panel borderColor: (Color black).
	panel setProperty: #objectName toValue: 'panel'.
	inputContainer := UiContainer new.
	inputContainer changeTableLayout.
	inputContainer extent: (210@22).
	inputContainer position: (4@4).
	inputContainer hResizing: (#spaceFill).
	inputContainer vResizing: (#shrinkWrap).
	inputContainer cellInset: (2).
	inputContainer cellPositioning: (#center).
	inputContainer cellSpacing: (#none).
	inputContainer layoutInset: (0).
	inputContainer listCentering: (#topLeft).
	inputContainer listDirection: (#leftToRight).
	inputContainer listSpacing: (#none).
	inputContainer maxCellSize: (1073741823).
	inputContainer minCellSize: (0).
	inputContainer wrapCentering: (#topLeft).
	inputContainer wrapDirection: (#none).
	inputContainer frameFractions: (0@0 corner: 1@1).
	inputContainer frameOffsets: (0@0 corner: 0@0).
	inputContainer balloonText: (nil).
	inputContainer balloonColor: ((TranslucentColor r: 0.91 g: 0.94 b: 1.0 alpha: 0.922)).
	inputContainer color: (Color white).
	inputContainer borderWidth: (1).
	inputContainer borderStyle2: (#inset).
	inputContainer borderColor: (#inset).
	inputContainer setProperty: #objectName toValue: 'inputContainer'.
	protocolBtn := UiToolButton new.
	protocolBtn changeTableLayout.
	protocolBtn extent: (69@19).
	protocolBtn position: (5@6).
	protocolBtn hResizing: (#shrinkWrap).
	protocolBtn vResizing: (#shrinkWrap).
	protocolBtn cellInset: (4).
	protocolBtn cellPositioning: (#center).
	protocolBtn cellSpacing: (#none).
	protocolBtn layoutInset: (2@0 corner: 2@0).
	protocolBtn listCentering: (#topLeft).
	protocolBtn listDirection: (#leftToRight).
	protocolBtn listSpacing: (#none).
	protocolBtn maxCellSize: (1073741823).
	protocolBtn minCellSize: (0).
	protocolBtn wrapCentering: (#topLeft).
	protocolBtn wrapDirection: (#none).
	protocolBtn frameFractions: (0@0 corner: 1@1).
	protocolBtn frameOffsets: (0@0 corner: 0@0).
	protocolBtn balloonText: ('').
	protocolBtn balloonColor: ((TranslucentColor r: 0.91 g: 0.94 b: 1.0 alpha: 0.941)).
	protocolBtn enabled: (true).
	protocolBtn setProperty: #iconByDesigner toValue: 'UiSilkIcons tagOrangeIcon'.
	protocolBtn icon: (UiSilkIcons tagOrangeIcon).
	protocolBtn text: ('protocol').
	protocolBtn color: (Color transparent).
	protocolBtn checkable: (false).
	protocolBtn checked: (false).
	protocolBtn autoExclusive: (false).
	protocolBtn autoRaise: (true).
	protocolBtn buttonStyle: (#textBesideIcon).
	protocolBtn setProperty: #objectName toValue: 'protocolBtn'.
	inputContainer addMorphBack: protocolBtn.
	searchField := UiLineEdit new.
	searchField extent: (113@20).
	searchField position: (76@5).
	searchField hResizing: (#spaceFill).
	searchField vResizing: (#rigid).
	searchField cellInset: (0).
	searchField cellPositioning: (#center).
	searchField cellSpacing: (#none).
	searchField layoutInset: (0).
	searchField listCentering: (#topLeft).
	searchField listDirection: (#topToBottom).
	searchField listSpacing: (#none).
	searchField maxCellSize: (1073741823).
	searchField minCellSize: (0).
	searchField wrapCentering: (#topLeft).
	searchField wrapDirection: (#none).
	searchField frameFractions: (0@0 corner: 1@1).
	searchField frameOffsets: (0@0 corner: 0@0).
	searchField balloonText: (nil).
	searchField balloonColor: ((TranslucentColor r: 0.91 g: 0.94 b: 1.0 alpha: 0.925)).
	searchField widgetResizable: (false).
	searchField horizontalScrollBarPolicy: (#alwaysOff).
	searchField verticalScrollBarPolicy: (#alwaysOff).
	searchField outerBorderWidth: (0).
	searchField theText: ('').
	searchField readOnly: (false).
	searchField autoConvert: (false).
	searchField autoAccept: (false).
	searchField acceptOnFocusLost: (true).
	searchField helpText: ('').
	searchField setProperty: #objectName toValue: 'searchField'.
	inputContainer addMorphBack: searchField.
	actionBtn := UiToolButton new.
	actionBtn changeTableLayout.
	actionBtn extent: (22@18).
	actionBtn position: (191@6).
	actionBtn hResizing: (#shrinkWrap).
	actionBtn vResizing: (#shrinkWrap).
	actionBtn cellInset: (4).
	actionBtn cellPositioning: (#center).
	actionBtn cellSpacing: (#none).
	actionBtn layoutInset: (2@0 corner: 2@0).
	actionBtn listCentering: (#topLeft).
	actionBtn listDirection: (#leftToRight).
	actionBtn listSpacing: (#none).
	actionBtn maxCellSize: (1073741823).
	actionBtn minCellSize: (0).
	actionBtn wrapCentering: (#topLeft).
	actionBtn wrapDirection: (#none).
	actionBtn frameFractions: (0@0 corner: 1@1).
	actionBtn frameOffsets: (0@0 corner: 0@0).
	actionBtn balloonText: (nil).
	actionBtn balloonColor: ((TranslucentColor r: 0.91 g: 0.94 b: 1.0 alpha: 0.922)).
	actionBtn enabled: (true).
	actionBtn setProperty: #iconByDesigner toValue: 'UiSilkIcons addIcon'.
	actionBtn icon: (UiSilkIcons addIcon).
	actionBtn text: ('ToolButton').
	actionBtn color: (Color transparent).
	actionBtn checkable: (false).
	actionBtn checked: (false).
	actionBtn autoExclusive: (false).
	actionBtn autoRaise: (true).
	actionBtn buttonStyle: (#iconOnly).
	actionBtn setProperty: #objectName toValue: 'actionBtn'.
	actionBtn setProperty: #objectDescription toValue: 'This button triggers the main action for the given text.'.
	inputContainer addMorphBack: actionBtn.
	panel addMorphBack: inputContainer.
	resultList := UiListWidget new.
	resultList extent: (210@70).
	resultList position: (4@26).
	resultList hResizing: (#spaceFill).
	resultList vResizing: (#rigid).
	resultList cellInset: (0).
	resultList cellPositioning: (#center).
	resultList cellSpacing: (#none).
	resultList layoutInset: (0).
	resultList listCentering: (#topLeft).
	resultList listDirection: (#topToBottom).
	resultList listSpacing: (#none).
	resultList maxCellSize: (1073741823).
	resultList minCellSize: (0).
	resultList wrapCentering: (#topLeft).
	resultList wrapDirection: (#none).
	resultList frameFractions: (0@0 corner: 1@1).
	resultList frameOffsets: (0@0 corner: 0@0).
	resultList balloonText: (nil).
	resultList balloonColor: ((TranslucentColor r: 0.91 g: 0.94 b: 1.0 alpha: 0.925)).
	resultList widgetResizable: (false).
	resultList horizontalScrollBarPolicy: (#alwaysOff).
	resultList verticalScrollBarPolicy: (#asNeeded).
	resultList outerBorderWidth: (0).
	resultList modelClass: (UiListWidgetModel).
	resultList horizontalHeaderVisible: (false).
	resultList verticalHeaderVisible: (false).
	resultList selectionBehavior: (#selectRows).
	resultList selectionColor: ((Color r: 0.772 g: 0.858 b: 1.0)).
	resultList selectionMode: (UiViewSingleSelection).
	resultList setProperty: #objectName toValue: 'resultList'.
	panel addMorphBack: resultList.
	panel setProperty: #uiClassName toValue: 'SdMethodFinder'.! !

!SdMethodFinderUi methodsFor: 'accessing' stamp: 'mt 4/14/2011 11:25:09.082'!
inputContainer
	"nil"
	
	^ inputContainer! !

!SdMethodFinderUi methodsFor: 'accessing' stamp: 'mt 4/14/2011 11:25:09.176'!
protocolBtn
	"nil"
	
	^ protocolBtn! !

!SdMethodFinderUi methodsFor: 'accessing' stamp: 'mt 4/14/2011 11:25:09.39'!
resultList
	"nil"
	
	^ resultList! !

!SdMethodFinderUi methodsFor: 'accessing' stamp: 'mt 4/14/2011 11:25:09.264'!
searchField
	"nil"
	
	^ searchField! !

Morph subclass: #SdClassElementEditor
	instanceVariableNames: 'theClass toolbar toolbarVisible outerBorderWidth'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sight-Dynamic'!

!SdClassElementEditor commentStamp: 'mt 4/6/2011 16:01:42.402' prior: 0!
This is an editor for a class element, e.g. a method, the comment...!

SdClassElementEditor subclass: #SdCallTreeView
	instanceVariableNames: 'ui cgt'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sight-Dynamic'!

!SdCallTreeView methodsFor: 'initialization' stamp: 'mt 4/17/2011 15:07:35.728'!
initialize

	super initialize.
	
	self ui setupUi: self.  
	
	self connect: self ui testClassCmb signal: #currentTextChanged: toSelector: #updateTestCombo:.
	self connect: self ui testCmb signal: #currentTextChanged: toSelector: #traceCurrentTest.
	
	self ui testClassCmb addTextList:
		((PackageInfo named: #'Widgets-Tests') classes
			select: [:class | class inheritsFrom: UiTestCase]
			thenCollect: [:uiTestClass | uiTestClass name]).! !

!SdCallTreeView methodsFor: 'tracing' stamp: 'mt 4/17/2011 15:08:25.147'!
traceCurrentTest

	| cgt tree testClass testMethodSelector |
	self ui callTree model: nil.
	
	testClass := Smalltalk at: self ui testClassCmb currentText asString asSymbol ifAbsent: [^ self].
	testMethodSelector := self ui testCmb currentText asString asSymbol.
	
	"cgt := SdCallgraphTracer new.
	cgt profilingMode: #ms.
	cgt selectedCategories: (
		(PackageInfo named: #Widgets) systemCategories,
		(PackageInfo named: #Designer) systemCategories,
		(PackageInfo named: #'Sight-Dynamic') systemCategories).
	cgt callgraphModel: SdCallgraph new.
	cgt traceTest: testClass method: testMethodSelector.
	
	self ui callTree
		model: (SdCallgraphModel new root: cgt callgraphModel);
		setHAlignmentForColumn: 2 to: #right."
	! !

!SdCallTreeView methodsFor: 'accessing' stamp: 'mt 4/17/2011 15:05:47.754'!
ui

	^ ui ifNil: [ui := SdCallTreePanelUi new]! !

!SdCallTreeView methodsFor: 'updating' stamp: 'mt 4/17/2011 15:07:54.714'!
updateTestCombo: aTestClassName

	self ui testCmb clear.

	(aTestClassName isNil or: [aTestClassName isEmpty])
		ifTrue: [^ self].

	self ui testCmb
		addTextList: (Smalltalk at: aTestClassName asString asSymbol) testSelectors.! !

SdClassElementEditor subclass: #SdClassEditor
	instanceVariableNames: 'ui sui finder commentEditor definitionEditor classFinder'
	classVariableNames: 'RecentlyUsedClasses'
	poolDictionaries: ''
	category: 'Sight-Dynamic'!

!SdClassEditor commentStamp: 'mt 4/6/2011 16:27:07.935' prior: 0!
This is an editor for a whole class.

SdClassEditor new
	width: 500;
	theClass: String;
	openInWorld.!

!SdClassEditor class methodsFor: 'accessing' stamp: 'mt 4/17/2011 12:20:45.46'!
addRecentlyUsedClass: className

	self recentlyUsedClasses remove: className ifAbsent: [].

	self recentlyUsedClasses addFirst: className.

	self recentlyUsedClasses size > 10
		ifTrue: [self recentlyUsedClasses removeLast].! !

!SdClassEditor class methodsFor: 'testing' stamp: 'mt 3/25/2011 17:05:25.965'!
isWidgetClass

	^ false! !

!SdClassEditor class methodsFor: 'accessing' stamp: 'mt 4/17/2011 12:13:47.564'!
recentlyUsedClasses

	^ RecentlyUsedClasses ifNil: [RecentlyUsedClasses := OrderedCollection new]! !

!SdClassEditor methodsFor: 'actions' stamp: 'mt 4/14/2011 19:20:52.215'!
activateFinder
	"Show the finder and give it the keyboard."
	
	self ui findBtn checked: true.
	finder	grabKeyboard.
	finder ui searchField selectAll.! !

!SdClassEditor methodsFor: 'actions - class' stamp: 'mt 4/17/2011 11:51:54.982'!
addClass: name category: category

	^ Object
		subclass: name
		instanceVariableNames: ''
		classVariableNames: ''
		poolDictionaries: ''
		category: category! !

!SdClassEditor methodsFor: 'initialization' stamp: 'mt 4/18/2011 14:24:46.643'!
addMethodEditor: aSelector isMeta: aBoolean

	| editor |
	editor := (self addMethodEditorSilently: aSelector isMeta: aBoolean)
		expand;
		grabKeyboard.
	
	self ui codeArea scrollToShow: editor bounds.
"	finder grabKeyboard."
	
	^ editor! !

!SdClassEditor methodsFor: 'initialization' stamp: 'mt 4/18/2011 15:47:08.821'!
addMethodEditor: aSelector isMeta: aBoolean protocol: protocolName
	"Classifies the selector under the given protocol name."
	
	| m organization |
	m := self addMethodEditor: aSelector isMeta: aBoolean.
	
	"Use method editor to retrieve the correct selector as the user may have entered a full method signature with arguments."
	organization := (aBoolean
		ifTrue: [self theClass theMetaClass]
		ifFalse: [self theClass]) organization.
		
	(organization categoryOfElement: m selector) ~= protocolName
		ifTrue: [organization classify: m selector under: protocolName].! !

!SdClassEditor methodsFor: 'initialization' stamp: 'mt 4/18/2011 15:41:49.12'!
addMethodEditorSilently: aSelector isMeta: aBoolean

	| m |
	self theClass ifNil: [^ self].

	(self methodEditors
		detect: [:methodEditor | methodEditor theClass isMeta = aBoolean
			and: [methodEditor selector = aSelector]]
		ifNone: []) ifNotNil: [:methodEditor | ^ methodEditor].

	m := SdMethodEditor new
			theClass: (aBoolean ifTrue: [self theClass theMetaClass] ifFalse: [self theClass])
			selector: aSelector;
			hResizing: #spaceFill.
			
	self editors addMorphBack: m.
		
	self connect: m signal: #focusChanged: toSelector: #onFocusChanged:.
	
	self connect: m sourceCodeEdit signal: #mouseWheelUp: to: self ui codeArea verticalScrollBar selector: #decreaseValueBySingleStep.
	self connect: m sourceCodeEdit signal: #mouseWheelDown: to: self ui codeArea verticalScrollBar selector: #increaseValueBySingleStep.
	m sourceCodeEdit eventHandler on: #(cmd) -> $F send: #activateFinder to: self.
	
	^ m! !

!SdClassEditor methodsFor: 'initialization' stamp: 'mt 4/18/2011 15:42:52.873'!
addMethodEditorsForProtocol: protocolName isMeta: aBoolean

	self theClass ifNil: [^ self].

	Cursor wait showWhile: [
		((aBoolean
			ifTrue: [self theClass theMetaClass organization]
			ifFalse: [self theClass organization])
			listAtCategoryNamed: protocolName) do: [:selector |
			(self addMethodEditorSilently: selector isMeta: aBoolean)]].
	
	finder grabKeyboard.! !

!SdClassEditor methodsFor: 'actions - class' stamp: 'mt 4/17/2011 12:22:11.105'!
chooseClass: className

	| class |
	(className isNil or: [className isEmpty]) ifTrue: [^ self].
	
	class := Smalltalk
		at: className
		ifAbsent: [self
			addClass: className
			category: 'Uncategorized'].	
			
	self theClass: class; switchToEditUi.! !

!SdClassEditor methodsFor: 'actions - class' stamp: 'mt 4/17/2011 11:54:09.3'!
chooseClass: className category: category

	| class |
	className ifEmpty: [^ self].
	
	class := Smalltalk
		at: className
		ifAbsent: [self
			addClass: className
			category: category].
		
	class category ~= category
		ifTrue: [class category: category].
			
	self theClass: class; switchToEditUi.! !

!SdClassEditor methodsFor: 'actions' stamp: 'mt 4/13/2011 15:56:04.971'!
closeAll

	self ui commentBtn checked: false.
	self ui definitionBtn checked: false.    
	
	self editors
		removeAllMorphs;
		height: 10.
	
	finder clear.
	self ui findBtn checked: true.! !

!SdClassEditor methodsFor: 'updating' stamp: 'mt 4/17/2011 12:23:54.216'!
editorChange: aSymbol

	aSymbol = #class
		ifTrue: [
			"Visually inform the user about the new class."
			self ui classLbl text: ('<b>', self theClass name, '</b>') asHtmlText.
			
			"Inform method finder about new class."
			self finder theClass: self theClass.
			
			"Reset comment editor."
			self loadClassComment.
			
			"Reset definition editor."
			definitionEditor theClass: self theClass.
			self loadClassDefinition.
			
			"Reset editors for new class."
			self editors removeAllMorphs.
			
			self class addRecentlyUsedClass: self theClass name.
			self updateRecentlyList].! !

!SdClassEditor methodsFor: 'accessing' stamp: 'mt 4/6/2011 16:13:52.204'!
editors
	"Container where all class element editors are stored."
	
	^ self ui codeArea widget! !

!SdClassEditor methodsFor: 'accessing' stamp: 'mt 4/6/2011 16:14:29.696'!
finder

	^ finder! !

!SdClassEditor methodsFor: 'testing' stamp: 'mt 4/14/2011 13:51:21.125'!
hasUnacceptedChanges

	^ definitionEditor hasUnacceptedChanges
		or: [commentEditor hasUnacceptedChanges]
		or: [self editors submorphs allSatisfy: [:e | e hasUnacceptedChanges]]! !

!SdClassEditor methodsFor: 'layouting' stamp: 'mt 4/11/2011 10:43:31.531'!
hideTopContainer

	self ui topContainer
		hide;
		disableTableLayout: true.! !

!SdClassEditor methodsFor: 'initialization' stamp: 'mt 4/17/2011 11:30:57.919'!
initialize

	super initialize.
	
	finder := SdMethodFinder new hResizing: #spaceFill.
	
	commentEditor := UiTextEdit new
		hResizing: #spaceFill;
		scrollBarThickness: 10.
	self connect: commentEditor signal: #editingFinished: toSelector: #saveClassComment.
	
	definitionEditor := SdClassDefinitionEdit new
		hResizing: #spaceFill;
		verticalScrollBarPolicy: #alwaysOff;  
		areaResizable: true.    
	self connect: definitionEditor signal: #editingFinished: toSelector: #saveClassDefinition.
	
	classFinder := SdClassFinder new hResizing: #spaceFill.  
	
	Cursor wait showWhile: [
		self
			initializeSystemChangeNotifications;
			initializeEditUi;
			initializeSelectUi;
			switchToSelectUi].! !

!SdClassEditor methodsFor: 'initialization' stamp: 'mt 4/18/2011 15:57:30.973'!
initializeEditUi

	self ui initializePanel.

	self ui codeArea verticalScrollBar singleStep: 20.
	
	self ui codeArea widget: (Morph new
		changeTableLayout;
		color: Color transparent;
		vResizing: #shrinkWrap;
		listDirection: #topToBottom).
	
	self ui topContainer addMorphBack: finder.
	self toggleFinder.
		
	self connect: self ui findBtn signal: #toggled: toSelector: #toggleFinder.
	self connect: finder signal: #deleted to: self ui findBtn selector: #checked: pattern: #(=false).
	
	self connect: finder signal: #selectorChosen:isMeta: toSelector: #addMethodEditor:isMeta:.
	self connect: finder signal: #selectorChosen:isMeta:protocol: toSelector: #addMethodEditor:isMeta:protocol:.
	
	self connect: self ui browseBtn signal: #clicked: toSelector: #switchToSelectUi.
	self connect: self ui closeBtn signal: #clicked: toSelector: #delete.
	self connect: self ui saveAllBtn signal: #clicked: toSelector: #saveAll.
	self connect: self ui closeAllBtn signal: #clicked: toSelector: #closeAll.
	
	self connect: self ui commentBtn signal: #toggled: toSelector: #toggleCommentEditor.
	self connect: self ui definitionBtn signal: #toggled: toSelector: #toggleDefinitionEditor.	! !

!SdClassEditor methodsFor: 'initialization' stamp: 'mt 4/14/2011 14:43:53.632'!
initializeEventHandler

	super initializeEventHandler.

	self eventHandler on: #(cmd) -> $F send: #activateFinder to: self.! !

!SdClassEditor methodsFor: 'initialization' stamp: 'mt 4/17/2011 15:04:43.265'!
initializeSelectUi

	| resizeHandle |

	self sui initializePanel.

	resizeHandle := SdResizeHandle new orientation: #horizontal.  
	resizeHandle target: classFinder ui resultList.
	self connect: classFinder signal: #deleted to: resizeHandle selector: #delete.
	self sui container
		addMorphBack: classFinder;
		addMorphBack: resizeHandle.  
	classFinder grabKeyboard.

	self sui recentlyList color: Color transparent.
		
	self connect: classFinder signal: #classChosen: toSelector: #chooseClass:.
	self connect: classFinder signal: #classChosen:category: toSelector: #chooseClass:category:.
	
	classFinder clear.
	
	self updateRecentlyList.
	
	self connect: self sui recentlyList signal: #currentTextChanged: toSelector: #chooseClass:.! !

!SdClassEditor methodsFor: 'initialization' stamp: 'mt 4/14/2011 18:26:51.125'!
initializeSystemChangeNotifications

	SystemChangeNotifier uniqueInstance 
		notify: self ofSystemChangesOfItem: SystemChangeNotifier methodKind using: #onMethodChange:;
		notify: self ofSystemChangesOfItem: SystemChangeNotifier classKind using: #onClassChange:;
		notify: self ofSystemChangesOfItem: SystemChangeNotifier protocolKind using: #onProtocolChange:;
		notify: self ofSystemChangesOfChange: SdMessageEvent changeKind using: #onMessageReceive:.! !

!SdClassEditor methodsFor: 'actions' stamp: 'mt 4/12/2011 14:16:14.727'!
loadClassComment

	commentEditor theText: self theClass organization classComment.
	self updateCommentBtn.! !

!SdClassEditor methodsFor: 'actions' stamp: 'mt 4/18/2011 16:34:50.914'!
loadClassDefinition

	definitionEditor theText:
		self theClass definition,
		'.', String cr,
		self theClass theMetaClass definition.! !

!SdClassEditor methodsFor: 'accessing' stamp: 'mt 4/6/2011 16:30:50.473'!
methodEditors

	^ self editors submorphs
		select: [:classElementEditor | classElementEditor isMethodEditor]! !

!SdClassEditor methodsFor: 'system changes' stamp: 'mt 4/18/2011 13:24:43.203'!
onClassChange: classKindEvent

	| locationString |
	classKindEvent itemClass theNonMetaClass ~= self theClass
		ifTrue: [^ self "ignore"].
	
	locationString := classKindEvent itemClass asString.
	classKindEvent isCommented
		ifTrue: [
			self showInformation: ('Comment changed.') asHtmlText.
			self loadClassComment].
		
	classKindEvent isModified
		ifTrue: [
			self showInformation: ('Definition changed.') asHtmlText.
			self loadClassDefinition].! !

!SdClassEditor methodsFor: 'focus' stamp: 'mt 4/8/2011 12:13:53.325'!
onFocusChanged: aGlobalRectangle

	self ui codeArea scrollToShow: 
		((self editors globalPointToLocal: aGlobalRectangle topLeft)
			extent: aGlobalRectangle extent).! !

!SdClassEditor methodsFor: 'system changes' stamp: 'mt 4/18/2011 13:24:35.628'!
onMessageReceive: messageEvent

	messageEvent itemClass theNonMetaClass ~= self theClass
		ifTrue: [^ self "ignore"].
	
	messageEvent isInformation ifTrue: [self showInformation: messageEvent text].
	messageEvent isWarning ifTrue: [self showWarning: messageEvent text].
	messageEvent isError ifTrue: [self showError: messageEvent text].! !

!SdClassEditor methodsFor: 'system changes' stamp: 'mt 4/18/2011 16:56:39.678'!
onMethodChange: methodKindEvent

	| locationString |
	methodKindEvent itemClass theNonMetaClass ~= self theClass
		ifTrue: [^ self "ignore"].
	
	locationString := '<b>', methodKindEvent itemSelector asString, '</b>'.
	methodKindEvent isModified
		ifTrue: [self showInformation: ('Modified ', locationString) asHtmlText].
	methodKindEvent isRemoved
		ifTrue: [self showInformation: ('Removed ', locationString) asHtmlText].
	methodKindEvent isAdded
		ifTrue: [self showInformation: ('Added ', locationString) asHtmlText].
	methodKindEvent isRecategorized
		ifTrue: [self showInformation: ('Recategorized ', locationString, ' into <b>', methodKindEvent itemProtocol, '</b>') asHtmlText].
	
	(self editors submorphs
		detect: [:m | m isMethodEditor
			and: [m selector = methodKindEvent itemSelector]
			and: [m theClass isMeta = methodKindEvent itemClass isMeta]]
		ifNone: []) ifNotNil: [:methodEditor |
			methodKindEvent isRemoved
				ifTrue: [methodEditor delete]
				ifFalse: [methodEditor editorChange: #method]].
	
	finder update.! !

!SdClassEditor methodsFor: 'system changes' stamp: 'mt 4/18/2011 13:24:59.455'!
onProtocolChange: protocolKindEvent

	protocolKindEvent itemClass theNonMetaClass ~= self theClass
		ifTrue: [^ self "ignore"].
		
	protocolKindEvent isAdded 
		ifTrue: [self showInformation: ('Protocol added: <b>', protocolKindEvent itemProtocol, '</b>') asHtmlText].
		
	finder update.! !

!SdClassEditor methodsFor: 'info messages' stamp: 'mt 4/10/2011 11:33:59.459'!
positionForNextMessage

	| corner |
	corner := self ui codeArea viewportBounds bottomRight.
	^ corner + (-1 @ 
			(((self submorphs
				select: [:m | m isKindOf: UiMessage]
				thenCollect: [:m | m top + 1]), {corner y}) min - corner y))! !

!SdClassEditor methodsFor: 'actions' stamp: 'mt 4/13/2011 16:08:46.536'!
saveAll

	commentEditor acceptChanges.
		
	self editors submorphs
		select: [:editor | editor isMethodEditor]
		thenDo: [:methodEditor | methodEditor saveMethod].! !

!SdClassEditor methodsFor: 'actions' stamp: 'mt 4/13/2011 15:46:01.523'!
saveClassComment

	self theClass organization classComment = commentEditor theText
		ifTrue: [^ self].

	self theClass comment: commentEditor theText.
	self updateCommentBtn.! !

!SdClassEditor methodsFor: 'actions' stamp: 'mt 4/12/2011 15:41:53.82'!
saveClassDefinition

! !

!SdClassEditor methodsFor: 'info messages' stamp: 'mt 4/8/2011 12:41:36.867'!
showError: aText
		
	self addMorph: ((UiMessage error: aText)
		disableTableLayout: true;  
		bottomRight: self positionForNextMessage).! !

!SdClassEditor methodsFor: 'info messages' stamp: 'mt 4/10/2011 11:26:35.321'!
showInformation: aText
		
	self addMorph: ((UiMessage information: aText)
		disableTableLayout: true;  
		bottomRight: self positionForNextMessage).! !

!SdClassEditor methodsFor: 'layouting' stamp: 'mt 4/11/2011 14:18:43.761'!
showTopContainer

	self ui topContainer
		show;
		disableTableLayout: false.! !

!SdClassEditor methodsFor: 'info messages' stamp: 'mt 4/8/2011 12:40:05.831'!
showWarning: aText
		
	self addMorph: ((UiMessage warning: aText)
		disableTableLayout: true;  
		bottomRight: self positionForNextMessage).! !

!SdClassEditor methodsFor: 'accessing' stamp: 'mt 4/11/2011 14:07:50.128'!
sui

	^ sui ifNil: [sui := SdClassSelectorUi new]! !

!SdClassEditor methodsFor: 'initialization' stamp: 'mt 4/15/2011 12:04:52.198'!
switchToEditUi

	self removeAllMorphs.
	self ui setupUi: self.
	
	self ui findBtn checked: true.
	finder
		clear;
		grabKeyboard.! !

!SdClassEditor methodsFor: 'initialization' stamp: 'mt 4/17/2011 12:07:03.515'!
switchToSelectUi

	self removeAllMorphs.
	self sui setupUi: self.
	
	classFinder grabKeyboard.! !

!SdClassEditor methodsFor: 'actions' stamp: 'mt 4/12/2011 15:31:33.22'!
toggleCommentEditor

	self ui commentBtn checked
		ifTrue: [
			self ui topContainer addMorphBack: commentEditor.
			commentEditor grabKeyboard]
		ifFalse: [commentEditor delete].! !

!SdClassEditor methodsFor: 'actions' stamp: 'mt 4/12/2011 15:31:45.865'!
toggleDefinitionEditor

	self ui definitionBtn checked
		ifTrue: [
			self ui topContainer addMorphBack: definitionEditor.
			definitionEditor grabKeyboard]
		ifFalse: [definitionEditor delete].! !

!SdClassEditor methodsFor: 'actions' stamp: 'mt 4/15/2011 10:22:56.489'!
toggleFinder

	self ui findBtn checked
		ifTrue: [ | resizeHandle |
			resizeHandle := SdResizeHandle new orientation: #horizontal.  
			resizeHandle target: finder ui resultList.
			self connect: finder signal: #deleted to: resizeHandle selector: #delete.
			self ui topContainer
				addMorphBack: finder;
				addMorphBack: resizeHandle.  
			finder grabKeyboard]
		ifFalse: [finder delete].! !

!SdClassEditor methodsFor: 'accessing' stamp: 'mt 3/29/2011 10:48:12.049'!
ui

	^ ui ifNil: [ui := SdClassEditorUi new]! !

!SdClassEditor methodsFor: 'updating' stamp: 'mt 4/12/2011 14:16:04.294'!
updateCommentBtn

	commentEditor theText
		ifEmpty: [self ui commentBtn icon: UiSilkIcons commentAddIcon]
		ifNotEmpty: [self ui commentBtn icon: UiSilkIcons commentIcon].! !

!SdClassEditor methodsFor: 'updating - classes' stamp: 'mt 4/17/2011 12:15:08.134'!
updateRecentlyList

	self sui recentlyList
		clear;
		addTextList: self class recentlyUsedClasses.! !

!SdClassElementEditor methodsFor: 'submorphs-add/remove' stamp: 'mt 4/13/2011 13:36:08.703'!
delete

	super delete.
	self deleted.! !

!SdClassElementEditor methodsFor: 'signals' stamp: 'mt 4/13/2011 13:36:19.879'!
deleted

	self emit.! !

!SdClassElementEditor methodsFor: 'layout' stamp: 'mt 4/7/2011 17:39:09.148'!
doLayoutIn: layoutBounds

	super doLayoutIn: layoutBounds.
	  
	"Update toolbar bounds."
	self toolbar ifNotNil: [:tb |
		tb bounds: ((self localPointToGlobal: self topLeft) extent: self bounds extent)].! !

!SdClassElementEditor methodsFor: 'drawing' stamp: 'mt 4/14/2011 17:07:03.228'!
drawOn: aCanvas

	super drawOn: aCanvas.
	
	self outerBorderWidth > 0 ifTrue: [
		aCanvas
			frameRoundRect: self outerBounds
			radius: 3
			width: self outerBorderWidth  
			color: (Color gray: 0.3)]  .! !

!SdClassElementEditor methodsFor: 'updating' stamp: 'mt 4/6/2011 16:06:40.85'!
editorChange: aSymbol
	"#class"
	
	! !

!SdClassElementEditor methodsFor: 'testing' stamp: 'mt 4/14/2011 13:50:06.099'!
hasUnacceptedChanges

	^ false! !

!SdClassElementEditor methodsFor: 'toolbar' stamp: 'mt 4/7/2011 17:16:16.644'!
hideToolbar

	self toolbarVisible ifFalse: [^ self].
	
	self toolbar delete.! !

!SdClassElementEditor methodsFor: 'initialization' stamp: 'mt 4/7/2011 17:10:50.53'!
initialize

	super initialize.
	
	self initializeEventHandler.! !

!SdClassElementEditor methodsFor: 'initialization' stamp: 'mt 4/14/2011 17:16:47.546'!
initializeEventHandler

	self eventHandler: UiEventHandler new.
	
	self connect: self eventHandler signal: #mouseEntered: toSelector: #showToolbar.! !

!SdClassElementEditor methodsFor: 'testing' stamp: 'mt 4/6/2011 16:23:21.581'!
isMethodEditor

	^ false! !

!SdClassElementEditor methodsFor: 'accessing' stamp: 'mt 4/14/2011 17:04:58.445'!
outerBorderWidth

	^ outerBorderWidth ifNil: [0]! !

!SdClassElementEditor methodsFor: 'accessing' stamp: 'mt 4/14/2011 17:05:10.521'!
outerBorderWidth: anInteger

	outerBorderWidth := anInteger.! !

!SdClassElementEditor methodsFor: 'geometry' stamp: 'mt 4/14/2011 17:05:24.67'!
outerBounds

	^ self bounds outsetBy: self outerBorderWidth  ! !

!SdClassElementEditor methodsFor: 'toolbar' stamp: 'mt 4/7/2011 17:15:22.705'!
showToolbar

	self toolbarVisible ifFalse: [^ self].
	
	self toolbar popUpAt: (self localPointToGlobal: self topLeft).! !

!SdClassElementEditor methodsFor: 'accessing' stamp: 'mt 4/6/2011 15:59:54.201'!
theClass

	^ theClass! !

!SdClassElementEditor methodsFor: 'accessing' stamp: 'mt 4/6/2011 16:07:02.498'!
theClass: aClassOrMetaClass

	theClass = aClassOrMetaClass ifTrue: [^ self].

	theClass := aClassOrMetaClass.
	
	self editorChange: #class.! !

!SdClassElementEditor methodsFor: 'accessing' stamp: 'mt 4/7/2011 17:05:54.576'!
toolbar

	^ toolbar! !

!SdClassElementEditor methodsFor: 'accessing' stamp: 'mt 4/7/2011 17:22:27.399'!
toolbar: aToolbar

	toolbar := aToolbar.
	toolbar ifNotNil: [self toolbarVisible: true].! !

!SdClassElementEditor methodsFor: 'accessing' stamp: 'mt 4/7/2011 17:07:39.175'!
toolbarVisible

	^ self toolbar notNil and: [toolbarVisible ifNil: [false]]! !

!SdClassElementEditor methodsFor: 'accessing' stamp: 'mt 4/7/2011 17:37:59.467'!
toolbarVisible: aBoolean

	toolbarVisible := aBoolean.
	
	aBoolean
		ifFalse: [self toolbar ifNotNil: [:tb | tb delete]].! !

SdClassElementEditor subclass: #SdClassElementFinder
	instanceVariableNames: 'ui category'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sight-Dynamic'!

!SdClassElementFinder class methodsFor: 'shortcuts' stamp: 'mt 4/17/2011 09:27:06.456'!
tokenForAllCategories

	^ ';'! !

!SdClassElementFinder class methodsFor: 'shortcuts' stamp: 'mt 4/17/2011 09:27:04.42'!
tokenForAllElements

	^ '.'! !

!SdClassElementFinder methodsFor: 'actions' stamp: 'mt 4/17/2011 09:21:38.102'!
addCategory

	self subclassResponsibility.! !

!SdClassElementFinder methodsFor: 'queries' stamp: 'mt 4/17/2011 10:14:57.794'!
allCategories

	^ self organization categories! !

!SdClassElementFinder methodsFor: 'queries' stamp: 'mt 4/17/2011 09:33:32.985'!
allElements

	self subclassResponsibility.! !

!SdClassElementFinder methodsFor: 'building' stamp: 'mt 4/17/2011 10:16:53.261'!
buildCategoryItemFor: aCategoryName

	self subclassResponsibility.! !

!SdClassElementFinder methodsFor: 'building' stamp: 'mt 4/17/2011 15:36:44.39'!
buildCategoryItemsFor: categories

	^ categories collect: [:category | self buildCategoryItemFor: category].! !

!SdClassElementFinder methodsFor: 'building' stamp: 'mt 4/17/2011 10:16:41.228'!
buildElementItemFor: anElementName

	self subclassResponsibility.! !

!SdClassElementFinder methodsFor: 'building' stamp: 'mt 4/17/2011 15:36:18.739'!
buildElementItemsFor: elements

	^ elements collect: [:element | self buildElementItemFor: element].! !

!SdClassElementFinder methodsFor: 'queries' stamp: 'mt 4/17/2011 11:09:28.629'!
categoriesFor: aCategoryName
	"Allow one level of subcategories."
	
	^ #()! !

!SdClassElementFinder methodsFor: 'accessing' stamp: 'mt 4/17/2011 09:10:54.787'!
category

	^ category! !

!SdClassElementFinder methodsFor: 'queries' stamp: 'mt 4/17/2011 16:43:48.188'!
category: aCategory matches: aToken

	^ aCategory startsWith: aToken  ! !

!SdClassElementFinder methodsFor: 'queries' stamp: 'mt 4/17/2011 17:04:24.828'!
categoryFor: anElementName

	^ self organization categoryOfElement: anElementName.! !

!SdClassElementFinder methodsFor: 'actions' stamp: 'mt 4/17/2011 11:15:22.559'!
chooseCategory

	category := self currentCategoryInList.
	
	self updateCategoryButton.
	
	self ui searchField theText: self class tokenForAllElements.
	self ui searchField selectAll.
	
	self updateResults.! !

!SdClassElementFinder methodsFor: 'actions' stamp: 'mt 4/17/2011 09:23:05.795'!
chooseElement

	self subclassResponsibility.! !

!SdClassElementFinder methodsFor: 'actions' stamp: 'mt 4/17/2011 11:12:50.55'!
chooseItem
	"Decide the next action depending on the current search token and the current protocol."

	self currentCategoryInList
		ifNotNil: [^ self chooseCategory].
		
	^ self chooseElement! !

!SdClassElementFinder methodsFor: 'actions' stamp: 'mt 4/17/2011 12:05:11.457'!
chooseNextResult

	self ui resultList isEmpty ifTrue: [^ self].
	
	self ui resultList currentRow
		ifNil: [self ui resultList currentRow: 1]
		ifNotNil: [self ui resultList currentRow:
			(self ui resultList currentRow + 1 min: self ui resultList size)].! !

!SdClassElementFinder methodsFor: 'actions' stamp: 'mt 4/17/2011 12:05:07.766'!
choosePrevResult

	self ui resultList isEmpty ifTrue: [^ self].
	
	self ui resultList currentRow
		ifNil: [self ui resultList currentRow: 1]
		ifNotNil: [self ui resultList currentRow:
			(self ui resultList currentRow - 1 max: 1)].! !

!SdClassElementFinder methodsFor: 'actions' stamp: 'mt 4/18/2011 14:14:30.365'!
clear

	category
		ifNotNil: [:cat | | matchedCategories |
			category := nil.
			self updateResults: self class tokenForAllCategories.

			"Select the current category in the list."
			matchedCategories := self ui resultList findItems: cat match: #matchStartsWith.
			matchedCategories
				ifEmpty: [self update]
				ifNotEmpty: [self ui resultList currentItem: matchedCategories first]]
		ifNil: [
			self ui searchField theText: self class tokenForAllCategories.
			self updateResults.
			self ui searchField selectAll.]



	
	
	! !

!SdClassElementFinder methodsFor: 'accessing' stamp: 'mt 4/17/2011 09:14:50.269'!
currentCategory

	category ifNotNil: [^ category].
	
	self ui resultList currentItem ifNil: [^ nil].
	
	^ self ui resultList currentItem
		valueOfProperty: #category
		ifAbsent: []! !

!SdClassElementFinder methodsFor: 'accessing' stamp: 'mt 4/17/2011 11:12:17.996'!
currentCategoryInList
	
	self ui resultList currentItem ifNil: [^ nil].
	
	^ self ui resultList currentItem
		valueOfProperty: #category
		ifAbsent: []! !

!SdClassElementFinder methodsFor: 'accessing' stamp: 'mt 4/17/2011 09:26:59.967'!
currentElement
		
	| token |
	token := self ui searchField theText asString.
	
	({self class tokenForAllElements. self class tokenForAllCategories} includes: token)
		ifTrue: [^ '']
		ifFalse: [^ token asSymbol].! !

!SdClassElementFinder methodsFor: 'updating' stamp: 'mt 4/17/2011 09:29:54.73'!
editorChange: aSymbol

	aSymbol = #class
		ifTrue: [self clear].! !

!SdClassElementFinder methodsFor: 'queries' stamp: 'mt 4/17/2011 16:43:58.319'!
element: anElement matches: aToken

	^ anElement startsWith: aToken  ! !

!SdClassElementFinder methodsFor: 'queries' stamp: 'mt 4/17/2011 10:06:01.159'!
elementsFor: aCategoryName

	^ self organization listAtCategoryNamed: aCategoryName  ! !

!SdClassElementFinder methodsFor: 'events' stamp: 'mt 4/17/2011 09:27:34.612'!
grabKeyboard

	self ui searchField grabKeyboard.! !

!SdClassElementFinder methodsFor: 'actions' stamp: 'mt 4/17/2011 09:29:31.283'!
hideCategory

	self ui protocolBtn delete.! !

!SdClassElementFinder methodsFor: 'actions' stamp: 'mt 4/17/2011 09:29:28.951'!
hideResults

	self ui resultList delete.! !

!SdClassElementFinder methodsFor: 'queries' stamp: 'mt 4/17/2011 09:38:33.333'!
iconFor: anElementName

	^ nil! !

!SdClassElementFinder methodsFor: 'icons' stamp: 'mt 4/17/2011 09:45:32.551'!
iconForCategory

	self subclassResponsibility.! !

!SdClassElementFinder methodsFor: 'icons' stamp: 'mt 4/17/2011 09:46:08.294'!
iconForCategoryAdd

	self subclassResponsibility.! !

!SdClassElementFinder methodsFor: 'icons' stamp: 'mt 4/17/2011 09:47:59.367'!
iconForCategoryEdit

	self subclassResponsibility.! !

!SdClassElementFinder methodsFor: 'initialization' stamp: 'mt 4/17/2011 10:26:19.941'!
initialize

	super initialize.

	self ui setupUi: self.
	self ui panel
		layoutInset: 0;  
		vResizing: #shrinkWrap.  
	
	self connect: self ui searchField signal: #textEdited: toSelector: #updateResults:.
	self connect: self ui searchField signal: #textEdited: toSelector: #updateSelection.
	
	self connect: self ui resultList signal: #currentTextChanged: toSelector: #updateSearchField.

	self connect: self ui searchField signal: #textChanged: toSelector: #updateActionButton.
	self connect: self ui searchField signal: #textChanged: toSelector: #updateCategoryButton.
	
	self connect: self ui actionBtn signal: #clicked: toSelector: #chooseItem.
	self connect: self ui searchField signal: #returnPressed: toSelector: #chooseItem.
	self connect: self ui resultList signal: #clicked: toSelector: #chooseItem.
	
	self connect: self ui searchField eventHandler signal: #mouseWheelUp: to: self ui resultList verticalScrollBar selector: #decreaseValueBySingleStep.
	self connect: self ui searchField eventHandler signal: #mouseWheelDown: to: self ui resultList verticalScrollBar selector: #increaseValueBySingleStep.
	
	"keyboard shortcuts"
	self ui searchField eventHandler on: Character arrowDown send: #chooseNextResult to: self.
	self ui searchField eventHandler on: Character arrowUp send: #choosePrevResult to: self.
	self ui searchField eventHandler on: Character escape send: #clear to: self.
	
	self
		color: (Color gray: 0.85);
		outerBorderWidth: 1;
		layoutInset: (0@1 corner: 0@0);
		vResizing: #shrinkWrap;  
		update.! !

!SdClassElementFinder methodsFor: 'testing' stamp: 'mt 4/17/2011 09:50:03.078'!
isCurrentElementNew

	self subclassResponsibility.! !

!SdClassElementFinder methodsFor: 'testing' stamp: 'mt 4/17/2011 10:01:02.529'!
isValid
	"Indicate whether calculcations, updates and signal emittations work properly. "
	
	^ true! !

!SdClassElementFinder methodsFor: 'accessing' stamp: 'mt 4/17/2011 10:04:06.659'!
organization

	^ self subclassResponsibility! !

!SdClassElementFinder methodsFor: 'accessing' stamp: 'mt 4/17/2011 11:34:58.615'!
preferredItemCount
	"Number of visible items before scrolling."
	
	^ 5! !

!SdClassElementFinder methodsFor: 'actions' stamp: 'mt 4/17/2011 09:29:25.131'!
showCategory

	self ui protocolBtn owner ifNotNil: [^ self].

	self ui inputContainer addMorphFront: self ui protocolBtn.! !

!SdClassElementFinder methodsFor: 'actions' stamp: 'mt 4/17/2011 11:35:13.237'!
showResults

	self ui resultList height: (self ui resultList defaultItemDelegate  defaultItemExtent y * (self ui resultList size min: self preferredItemCount) + 3).
	
	self ui resultList owner ifNotNil: [^ self].
	
	self ui panel addMorphBack: self ui resultList.! !

!SdClassElementFinder methodsFor: 'accessing' stamp: 'mt 4/17/2011 09:12:59.695'!
ui

	^ ui ifNil: [ui := SdMethodFinderUi new]! !

!SdClassElementFinder methodsFor: 'updating' stamp: 'mt 4/17/2011 09:30:44.872'!
update

	self
		updateResults;
		updateSelection;
		updateActionButton;
		updateCategoryButton.! !

!SdClassElementFinder methodsFor: 'updating' stamp: 'mt 4/17/2011 10:02:05.212'!
updateActionButton

	self isValid
		ifFalse: [^ self ui actionBtn hide]
		ifTrue: [self ui actionBtn show].
	
	self currentElement ifEmpty: [^ self ui actionBtn hide].
	
	(self currentCategory notNil and: [category isNil]) ifTrue: [
		^ self ui actionBtn icon: self iconForCategoryEdit].
	
	self isCurrentElementNew
		ifFalse: [self ui actionBtn icon: UiSilkIcons pencilIcon]
		ifTrue: [
			({self class tokenForAllElements. self class tokenForAllCategories}
				includes: self currentElement)
					ifTrue: [self ui actionBtn hide]
					ifFalse: [	self ui actionBtn icon: UiFugueIcons asteriskYellowIcon]].! !

!SdClassElementFinder methodsFor: 'updating' stamp: 'mt 4/17/2011 09:47:46.568'!
updateCategoryButton

	self ui protocolBtn disconnectSignal: #clicked: from: self.

	category
		ifNil: [
			self hideCategory.
			
			(self currentCategory isNil and: [self currentElement notEmpty]) ifTrue: [
				"Show protocol button as action to create a new protocol."
				self ui inputContainer
					addMorph: self ui protocolBtn
					inFrontOf: self ui actionBtn.
				self ui protocolBtn
					icon: self iconForCategoryAdd;
					buttonStyle: #iconOnly;
					balloonText: 'Add new category'.
				self connect: self ui protocolBtn signal: #clicked: toSelector: #addCategory.]]
		ifNotNil: [
			self showCategory.
			
			self ui protocolBtn
				icon: self iconForCategory;
				buttonStyle: #textBesidesIcon;
				text: ('<font color="#AAAAAA">', category, '</font>') asHtmlText;
				balloonText: 'Discard filter'.
				
			self connect: self ui protocolBtn signal: #clicked: toSelector: #clear.]! !

!SdClassElementFinder methodsFor: 'updating - results' stamp: 'mt 4/17/2011 09:56:04.308'!
updateItemsForResults
	"Icons or text or items may have to be updated."
	
	self ui resultList items do: [:listWidgetItem |
		(listWidgetItem includesProperty: #element)
			ifTrue: [ | s i t |
				s := listWidgetItem valueOfProperty: #element.
				i := self iconFor: s.
				t := self category ifNil: [(s escapeHtmlText,
					' <font color="#AAAAAA">(',
					(self categoryFor: s),
					')</font>') asHtmlText] ifNotNil: [s].
					
				i ~~ listWidgetItem icon ifTrue: [listWidgetItem icon: i].
				t ~= listWidgetItem text ifTrue: [listWidgetItem text: t]]].! !

!SdClassElementFinder methodsFor: 'updating - results' stamp: 'mt 4/17/2011 09:41:20.518'!
updateResults

	self updateResults: self ui searchField theText.  ! !

!SdClassElementFinder methodsFor: 'updating - results' stamp: 'mt 4/17/2011 16:44:25.025'!
updateResults: stringOrText

	| token elements categories |
	token := stringOrText asString asSymbol.

	(self isValid not or: [token isEmpty]) ifTrue: [
		self ui resultList clear.
		^ self hideResults].

	"Find all elements, consider the current category and search token."
	elements := ((category
		ifNil: [self allElements]
		ifNotNil: [:cat | self elementsFor: cat])
		select: [:selector | token = self class tokenForAllElements or: [self element: selector matches: token]]) sorted.

	"Add categories to result if no current category is set."
	categories := ((category
		ifNil: [self allCategories]
		ifNotNil: [self categoriesFor: category])
		select: [:cat | token = self class tokenForAllCategories or: [self category: cat matches: token]]) sorted.
	
	"Only update the list if any changes occured."
	categories, elements = (self ui resultList items
		"Use extra property and not the text of the item to allow more rich information to be displayed, e.g., the category for an element."
		collect: [:item | item valueOfProperty: #symbol]) asArray
		ifTrue: [^ self updateItemsForResults].
	
	"Discard the old contents of the list because all new contents have been gathered."
	self ui resultList clear.
	
	self ui resultList
		addItemList: (self buildCategoryItemsFor: categories);
		addItemList: (self buildElementItemsFor: elements).
	
	self ui resultList isEmpty
		ifTrue: [self hideResults]
		ifFalse: [self showResults].! !

!SdClassElementFinder methodsFor: 'updating' stamp: 'mt 4/17/2011 09:44:46.001'!
updateSearchField
	"Completes the search field with the current text in the result list. Select that part that has not been typed yet."

	| userText currentItemToken currentSearchToken i |
	self ui resultList currentItem ifNil: [^ self].
	
	currentItemToken := (self ui resultList currentItem valueOfProperty: #symbol) asString.
	currentSearchToken := self ui searchField theText asString.

	currentItemToken = currentSearchToken 
		ifTrue: [^ self].
	
	userText := currentSearchToken allButLast: self ui searchField selectedText size.
	
	(currentItemToken startsWith: userText)
		ifTrue: [i := userText size + 1].

	self ui searchField theText: currentItemToken.
	
	i
		ifNil: [self ui searchField selectAll]
		ifNotNil: [self ui searchField select: (i to: self ui searchField theText size)].
			
	self grabKeyboard.
! !

!SdClassElementFinder methodsFor: 'updating' stamp: 'mt 4/17/2011 09:44:02.512'!
updateSelection
	"Try to find an exact match in the result list and select that item."
	
	self ui resultList currentItem: (self ui resultList items
		detect: [:item | (item valueOfProperty: #symbol) = self currentElement]
		ifNone: []).! !

SdClassElementFinder subclass: #SdClassFinder
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sight-Dynamic'!

!SdClassFinder commentStamp: 'mt 4/17/2011 16:30:36.489' prior: 0!
Class hierarchy sorting ported from OmniBrowser framework (OBClassSortFilter).!

!SdClassFinder methodsFor: 'actions' stamp: 'mt 4/17/2011 10:06:54.394'!
addCategory

	self currentElement ifEmpty: [^ self].
	
	self organization addCategory: self currentElement.! !

!SdClassFinder methodsFor: 'queries' stamp: 'mt 4/17/2011 11:08:07.496'!
allCategories

	^ (super allCategories, PackageOrganizer default packageNames) asSet! !

!SdClassFinder methodsFor: 'queries' stamp: 'mt 4/17/2011 10:08:58.07'!
allElements

	^ Smalltalk globals classNames! !

!SdClassFinder methodsFor: 'building' stamp: 'mt 4/17/2011 10:56:01.42'!
buildCategoryItemFor: aCategoryName

	| isPackage text |
	isPackage := PackageOrganizer default packageNames includes: aCategoryName.
	text := '{1} <font color="#AAAAAA">({2} class(es){3})</font>' format: {
		aCategoryName.
		(self elementsFor: aCategoryName) size.
		isPackage ifTrue:[', ', (PackageInfo named: aCategoryName) systemCategories size asString, ' categories'] ifFalse: ['']}.
	
	^ UiListWidgetItem new
		text: text asHtmlText;
		icon: (isPackage
			ifTrue: [UiSilkIcons packageIcon]
			ifFalse: [self iconForCategory]);
		setProperty: #category toValue: aCategoryName;
		setProperty: #symbol toValue: aCategoryName! !

!SdClassFinder methodsFor: 'building' stamp: 'mt 4/17/2011 15:39:53.453'!
buildElementItemFor: anElementName

	| isPackage |
	isPackage := PackageOrganizer default packageNames includes: self category.

	^ UiListWidgetItem new
		text: ((isPackage or: [self category isNil])
			ifTrue: [(anElementName escapeHtmlText, ' <font color="#AAAAAA">(', (self categoryFor: anElementName), ')</font>') asHtmlText]
			ifFalse: [anElementName]);
		"icon: (self iconFor: anElementName);"
		setProperty: #element toValue: anElementName;
		setProperty: #symbol toValue: anElementName! !

!SdClassFinder methodsFor: 'building' stamp: 'mt 4/17/2011 16:27:00.534'!
buildElementItemsFor: sortedElements
	"Sorting ported from OmniBrowser code."
	
	| classes order superChain |
	self category ifNil: [^ super buildElementItemsFor: sortedElements].
	
	classes := sortedElements collect: [:e | Smalltalk at: e].
	order := SortedCollection sortBlock: 
					[:assoc1 :assoc2 | | i chain2 chain1 result max |
					result := nil.
					chain1 := assoc1 key.
					chain2 := assoc2 key.
					max := chain1 size min: chain2 size.
					i := 1.
					[result isNil and: [i <= max]] whileTrue: 
							[(chain1 at: i) = (chain2 at: i) 
								ifTrue: [i := i + 1]
								ifFalse: [result := (chain1 at: i) name < (chain2 at: i) name]].
					result isNil ifTrue: [chain1 size < chain2 size] ifFalse: [result]].

	classes do: [:class |
		superChain := class withAllSuperclasses reversed.
		superChain removeAllSuchThat: [:cl | (classes includes: cl) not].
		order add: superChain contents -> class].
	
	^ order collect: [:assoc | 
			(self buildElementItemFor: assoc value name)
				icon: (assoc key size > 1
					ifTrue: [Form extent: (10 * (assoc key size - 1)) @1 depth: 32]
					ifFalse: [])].! !

!SdClassFinder methodsFor: 'queries' stamp: 'mt 4/17/2011 11:25:26.221'!
categoriesFor: aPackageName


	^ (PackageOrganizer default packageNamed: aPackageName ifAbsent: [])
		ifNil: [#()]
		ifNotNil: [:packageInfo | packageInfo systemCategories]! !

!SdClassFinder methodsFor: 'actions' stamp: 'mt 4/18/2011 12:01:45.116'!
chooseElement

	self currentElement ifEmpty: [^ self].

	"Category/package only relevant for new elements."
	self isCurrentElementNew
		ifTrue: [category
			ifNil: [^ self classChosen: self currentElement]
			ifNotNil: [^ self classChosen: self currentElement category: category]].

	self classChosen: self currentElement.! !

!SdClassFinder methodsFor: 'signals' stamp: 'mt 4/17/2011 10:09:39.977'!
classChosen: className

	self emit.! !

!SdClassFinder methodsFor: 'signals' stamp: 'mt 4/17/2011 10:09:50.735'!
classChosen: className category: aSystemCategory

	self emit.! !

!SdClassFinder methodsFor: 'actions' stamp: 'mt 4/17/2011 12:02:55.259'!
clear
	"Just go back to sys cat list in package if package can be found."
	
	| sysCat package matchedCategories |
	sysCat := category.
	
	package := PackageOrganizer default
		packageOfSystemCategory: category
		ifNone: [].
		
	(package isNil or: [package name = category]) ifTrue: [^ super clear].
	
	category := package name.
	self updateResults: self class tokenForAllCategories.
	
	"Select the current category in the list."
	matchedCategories := self ui resultList findItems: sysCat match: #matchStartsWith.
	matchedCategories ifNotEmpty: [self ui resultList currentItem: matchedCategories first]! !

!SdClassFinder methodsFor: 'accessing' stamp: 'mt 4/17/2011 15:28:33.834'!
currentPackage

	^ PackageOrganizer default
		packageNamed: self category
		ifAbsent: []! !

!SdClassFinder methodsFor: 'queries' stamp: 'mt 4/17/2011 16:51:17.362'!
element: anElement matches: aToken

	| letters |
	(super element: anElement matches: aToken)
		ifTrue: [^ true].
		
	letters := anElement select: [:e | e isUppercase].
	^ letters includesSubString: aToken  
	! !

!SdClassFinder methodsFor: 'queries' stamp: 'mt 4/17/2011 11:00:24.665'!
elementsFor: aCategoryName

	(PackageOrganizer default packageNames includes: aCategoryName)
		ifFalse: [^ super elementsFor: aCategoryName]
		ifTrue: [^ (PackageInfo named: aCategoryName) systemCategories gather:
			[:cat | (SystemOrganization listAtCategoryNamed: cat)]]! !

!SdClassFinder methodsFor: 'icons' stamp: 'mt 4/17/2011 10:36:08.29'!
iconForCategory

	^ UiSilkIcons folderIcon! !

!SdClassFinder methodsFor: 'icons' stamp: 'mt 4/17/2011 10:36:47.331'!
iconForCategoryAdd

	^ UiSilkIcons folderAddIcon! !

!SdClassFinder methodsFor: 'icons' stamp: 'mt 4/17/2011 10:36:56.874'!
iconForCategoryEdit

	^ UiSilkIcons folderEditIcon! !

!SdClassFinder methodsFor: 'testing' stamp: 'mt 4/17/2011 10:14:15.233'!
isCurrentElementNew

	^ (Smalltalk includesKey: self currentElement) not! !

!SdClassFinder methodsFor: 'accessing' stamp: 'mt 4/17/2011 10:06:20.147'!
organization

	^ SystemOrganization! !

!SdClassFinder methodsFor: 'accessing' stamp: 'mt 4/17/2011 11:35:21.882'!
preferredItemCount

	^ 15! !

!SdClassFinder methodsFor: 'updating' stamp: 'mt 4/17/2011 15:30:46.864'!
updateCategoryButton

	super updateCategoryButton.

	self currentPackage
		ifNotNil: [self ui protocolBtn icon: UiSilkIcons packageIcon].! !

SdClassElementFinder subclass: #SdMethodFinder
	instanceVariableNames: 'sideBtn'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sight-Dynamic'!

SdMethodFinder class
	instanceVariableNames: 'iconCache'!

SdMethodFinder class
	instanceVariableNames: 'iconCache'!

!SdMethodFinder class methodsFor: 'custom icons' stamp: 'mt 4/14/2011 20:00:18.291'!
arrowUpDownIcon

	^ self icons
		at: #arrowUpDown
		ifAbsentPut: [
			| form canvas |
			form := Form extent: 16@16 depth: 32.
			canvas := form getCanvas.
			
			canvas
				translucentImage: UiFugueIcons control270SmallIcon
				at: 0 @ 3;
				translucentImage: UiFugueIcons control090SmallIcon
				at: 0 @ -3.
				
			form]! !

!SdMethodFinder class methodsFor: 'custom icons' stamp: 'mt 4/14/2011 20:09:44.359'!
blankIcon

	^ self icons
		at: #blank
		ifAbsentPut: [Form extent: 16@16 depth: 32].! !

!SdMethodFinder class methodsFor: 'method icons' stamp: 'mt 4/14/2011 20:11:59.828'!
iconForClass: aClassDescription selector: aSelector
	"Answer an icon to be shown for the selector aSymbol in aClassDescription."

	aClassDescription methodDictionary at: aSelector ifPresent: [ :method |
		method literalsDo: [ :literal |
			(literal == #halt or: [ literal == #halt: or: [ literal == #haltIfNil or: [ literal == #haltIf: or: [ literal == #haltOnce or: [ literal == #haltOnce: or: [ literal == #haltOnCount: or: [ literal == #halt:onCount: ] ] ] ] ] ] ])
				ifTrue: [ ^ UiSilkIcons exclamationIcon].
			(literal == #flag: or: [ literal == #needsWork or: [ literal == #notYetImplemented ] ])
				ifTrue: [ ^ UiSilkIcons flagGreenIcon].
			"BEGIN: Signal extension"
			literal == #emit
				ifTrue: [^ UiSilkIcons transmitIcon].
			literal == #emitAlways
				ifTrue: [^ UiSilkIcons transmitBlueIcon].
			(literal == #shouldBeImplemented or: [literal == #subclassResponsibility])
				ifTrue: [^ UiFugueIcons controlStopSquareSmallIcon]
			"END: Signal extension" ] ].
	^ (aClassDescription isOverride: aSelector)
		ifTrue: [
			(aClassDescription isOverridden: aSelector)
				ifTrue: [ self arrowUpDownIcon ]
				ifFalse: [ UiFugueIcons control090SmallIcon ] ]
		ifFalse: [
			(aClassDescription isOverridden: aSelector)
				ifTrue: [ UiFugueIcons control270SmallIcon ]
				ifFalse: [ self blankIcon ] ]! !

!SdMethodFinder class methodsFor: 'custom icons' stamp: 'mt 4/14/2011 19:51:48.27'!
icons

	^ iconCache ifNil: [iconCache := Dictionary new]! !

!SdMethodFinder methodsFor: 'actions' stamp: 'mt 4/18/2011 15:47:23.429'!
addCategory

	self currentElement ifEmpty: [^ self].
	
	self organization addCategory: self currentElement.
	
	"TODO: Move notification into class categorizer."
	SystemChangeNotifier uniqueInstance trigger: ((AddedEvent
		item: self currentElement
		kind: AbstractEvent protocolKind)
		itemClass: self theClass).! !

!SdMethodFinder methodsFor: 'queries' stamp: 'mt 4/17/2011 09:34:14.123'!
allElements

	^ self theClass selectors! !

!SdMethodFinder methodsFor: 'building' stamp: 'mt 4/17/2011 10:17:50.949'!
buildCategoryItemFor: aCategoryName

	^ UiListWidgetItem new
		text: (aCategoryName, ' <font color="#AAAAAA">(', (self elementsFor: aCategoryName) size asString, ' method(s))</font>') asHtmlText;
		icon: self iconForCategory;
		setProperty: #category toValue: aCategoryName;
		setProperty: #symbol toValue: aCategoryName! !

!SdMethodFinder methodsFor: 'building' stamp: 'mt 4/17/2011 10:18:43.438'!
buildElementItemFor: anElementName

	^ UiListWidgetItem new
		text: (self category
			ifNil: [(anElementName escapeHtmlText, ' <font color="#AAAAAA">(', (self categoryFor: anElementName), ')</font>') asHtmlText]
			ifNotNil: [anElementName]);
		icon: (self iconFor: anElementName);
		setProperty: #element toValue: anElementName;
		setProperty: #symbol toValue: anElementName! !

!SdMethodFinder methodsFor: 'actions' stamp: 'mt 4/18/2011 14:23:44.072'!
chooseElement

	self currentElement ifEmpty: [^ self].
	
	self isCurrentElementNew
		ifTrue: [category
			ifNil: [^ self selectorChosen: self currentElement isMeta: self isMeta]
			ifNotNil: [^ self selectorChosen: self currentElement isMeta: self isMeta protocol: category]].
	
	self selectorChosen: self currentElement isMeta: self isMeta.! !

!SdMethodFinder methodsFor: 'queries' stamp: 'mt 4/17/2011 09:38:52.26'!
iconFor: anElementName

	^ self class iconForClass: self theClass selector: anElementName! !

!SdMethodFinder methodsFor: 'icons' stamp: 'mt 4/17/2011 09:46:02.656'!
iconForCategory

	^ UiSilkIcons tagOrangeIcon! !

!SdMethodFinder methodsFor: 'icons' stamp: 'mt 4/17/2011 09:46:15.175'!
iconForCategoryAdd

	^ UiSilkIcons tagBlueAddIcon! !

!SdMethodFinder methodsFor: 'icons' stamp: 'mt 4/17/2011 09:48:09.681'!
iconForCategoryEdit

	^ UiSilkIcons tagBlueEditIcon! !

!SdMethodFinder methodsFor: 'initialization' stamp: 'mt 4/18/2011 16:40:10.427'!
initialize

	super initialize.
	
	sideBtn := UiToolButton new
		text: #instance;
		icon: SdIcons instanceSideIcon;
		buttonStyle: #iconOnly;
		autoRaise: true;
		color: Color transparent;
		layoutInset: 0.
		
	self ui inputContainer addMorphFront: sideBtn.
	
	self connect: sideBtn signal: #clicked: toSelector: #toggleSide.
	self ui searchField eventHandler on: #(cmd) -> $q send: #toggleSide to: self.! !

!SdMethodFinder methodsFor: 'testing' stamp: 'mt 4/18/2011 14:21:27.112'!
isCurrentElementNew

	^ (self theClass includesSelector: self currentElement) not! !

!SdMethodFinder methodsFor: 'testing' stamp: 'mt 4/18/2011 14:05:08.566'!
isMeta

	sideBtn ifNil: [^ false].
	
	^ sideBtn text = #class! !

!SdMethodFinder methodsFor: 'testing' stamp: 'mt 4/17/2011 10:01:22.085'!
isValid
	"The method finder needs a class."
	
	^ self theClass notNil! !

!SdMethodFinder methodsFor: 'accessing' stamp: 'mt 4/18/2011 14:17:58.907'!
organization

	^ self theClass organization! !

!SdMethodFinder methodsFor: 'signals' stamp: 'mt 4/18/2011 13:03:31.962'!
selectorChosen: aSymbol isMeta: aBoolean
	"Emitted whenever the user decided for a symbol. This can be a new one."
	
	self emit.! !

!SdMethodFinder methodsFor: 'signals' stamp: 'mt 4/18/2011 13:04:03.486'!
selectorChosen: aSymbol isMeta: aBoolean protocol: protocolName
	"Emitted whenever the user decided for a symbol. This can be a new one."
		
	self emit.! !

!SdMethodFinder methodsFor: 'actions' stamp: 'mt 4/18/2011 13:59:35.056'!
showCategory

	self ui protocolBtn owner ifNotNil: [^ self].

	"Side button should always be the first one."
	self ui inputContainer addMorph: self ui protocolBtn after: sideBtn.! !

!SdMethodFinder methodsFor: 'accessing' stamp: 'mt 4/18/2011 14:19:36.992'!
theClass

	super theClass ifNil: [^ nil].

	^ self isMeta
		ifTrue: [super theClass theMetaClass]
		ifFalse: [super theClass]! !

!SdMethodFinder methodsFor: 'actions' stamp: 'mt 4/18/2011 14:02:00.607'!
toggleSide

	self isMeta
		ifTrue: [sideBtn text: #instance]
		ifFalse: [sideBtn text: #class].
		
	self
		updateSideBtn;
		clear.! !

!SdMethodFinder methodsFor: 'updating' stamp: 'mt 4/18/2011 14:09:17.688'!
updateSideBtn

	self isMeta
		ifTrue: [sideBtn
			icon: SdIcons classSideIcon;
			balloonText: 'class side - click to toggle'.]
		ifFalse: [sideBtn
			icon: SdIcons instanceSideIcon;
			balloonText: 'instance side - click to toggle'].! !

SdClassElementEditor subclass: #SdMethodEditor
	instanceVariableNames: 'sourceCodeEdit summaryContainer toggleBtn separator deleteBtn saveBtn revertBtn'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sight-Dynamic'!

!SdMethodEditor methodsFor: 'actions' stamp: 'mt 4/11/2011 11:26:23.287'!
collapse

	toggleBtn checked: false.! !

!SdMethodEditor methodsFor: 'submorphs-add/remove' stamp: 'mt 4/11/2011 11:32:12.156'!
delete

	super delete.
	self toolbar delete.! !

!SdMethodEditor methodsFor: 'actions' stamp: 'mt 4/12/2011 17:31:36.671'!
deleteMethod

	deleteBtn buttonStyle = #iconOnly
		"User needs to confirm deletion!!"
		ifTrue: [
			deleteBtn buttonStyle: #textBesideIcon.
			self toolbar updateLayout. "new toolbar extent"
			^ self]  
		"User confirmed the deletion..."
		ifFalse: [
			self theClass removeSelector: self selector.
			deleteBtn buttonStyle: #iconOnly.
			self delete].! !

!SdMethodEditor methodsFor: 'updating' stamp: 'mt 4/12/2011 16:21:59.105'!
editorChange: aSymbol
	
	sourceCodeEdit reset.
	self updateLayout.! !

!SdMethodEditor methodsFor: 'actions' stamp: 'mt 4/6/2011 16:36:26.142'!
expand

	toggleBtn checked: true.  ! !

!SdMethodEditor methodsFor: 'signals' stamp: 'mt 4/8/2011 11:19:22.195'!
focusChanged: aRectangle

	self emitAlways.  ! !

!SdMethodEditor methodsFor: 'events' stamp: 'mt 4/12/2011 17:53:48.58'!
grabKeyboard

	sourceCodeEdit grabKeyboard.! !

!SdMethodEditor methodsFor: 'testing' stamp: 'mt 4/14/2011 13:51:34.921'!
hasUnacceptedChanges

	^ sourceCodeEdit hasUnacceptedChanges! !

!SdMethodEditor methodsFor: 'initialization' stamp: 'mt 4/12/2011 17:56:46.268'!
initialize

	super initialize.

	sourceCodeEdit := SdSourceCodeEdit new
		hResizing: #spaceFill.
	"sourceCodeEdit textMorph margins: (20@0 corner: 3@0)."
	sourceCodeEdit
		verticalScrollBarPolicy: #alwaysOff;
		areaResizable: true;
		outerBorderWidth: 0.
	self connect: sourceCodeEdit signal: #focusChanged: toSelector: #focusChanged:.

	self connect: sourceCodeEdit signal: #editingFinished: toSelector: #updateToolbarButtons.
	self connect: sourceCodeEdit signal: #textChanged: toSelector: #updateToolbarButtons.

	self
		initializeToolbar;
		initializeSummaryContainer.

	separator := Morph new
		extent: 10@1;
		color: Color black;
		hResizing: #spaceFill.
		
	self
		color: Color transparent;
		changeTableLayout;
		listDirection: #topToBottom;
		vResizing: #shrinkWrap;  
		addMorphBack: summaryContainer;
		addMorphBack: sourceCodeEdit;
		addMorphBack: separator;
		updateLayout.
		
"	self connect: sourceCodeEdit signal: #sourceCodeResetted toSelector: #updateLayout."! !

!SdMethodEditor methodsFor: 'initialization' stamp: 'mt 4/12/2011 16:50:54.642'!
initializeSummaryContainer
	"That container is visible of the editor is collapsed."

	| closeBtn |
	summaryContainer := Morph new
		color: (Color gray: 0.95);
		changeTableLayout;
		listDirection: #leftToRight;
		vResizing: #shrinkWrap.  
	
	toggleBtn := UiToolButton new
		color: (Color gray: 0.95);
		autoRaise: true;  
		checkable: true;
		hResizing: #spaceFill.
	
	closeBtn := UiToolButton new
		color: (Color gray: 0.95);
		autoRaise: true;  
		buttonStyle: #iconOnly;
		icon: UiSilkIcons deleteIcon.
	
	summaryContainer
		addMorphBack: toggleBtn;
		addMorphBack: closeBtn.
	
	self connect: toggleBtn signal: #toggled: toSelector: #updateLayout.
	self connect: toggleBtn signal: #clicked: to: self selector: #showToolbar.
	self connect: closeBtn signal: #clicked: toSelector: #delete.! !

!SdMethodEditor methodsFor: 'initialization' stamp: 'mt 4/13/2011 15:33:16.89'!
initializeToolbar

	| closeBtn collapseBtn |
	self toolbar: SdFadingToolbar new.
	
	collapseBtn := UiToolButton new
		autoRaise: true;
		color: Color transparent;  
		buttonStyle: #iconOnly;
		icon: UiSilkIcons eyeIcon;
		balloonText: 'Collapse method editor'.
	self connect: collapseBtn signal: #clicked: toSelector: #collapse.
	
	self toolbar addButton: collapseBtn edge: #top.

	closeBtn := UiToolButton new
		autoRaise: true;
		color: Color transparent;  
		buttonStyle: #iconOnly;
		icon: UiSilkIcons deleteIcon;
		balloonText: 'Close method editor'.
	self connect: closeBtn signal: #clicked: toSelector: #delete.
	
	self toolbar addButton: closeBtn edge: #top.
		
	saveBtn := UiToolButton new
		autoRaise: true;
		color: Color transparent;  
		buttonStyle: #iconOnly;
		icon: UiSilkIcons diskIcon;
		balloonText: 'Save changed source'.
	self connect: saveBtn signal: #clicked: toSelector: #saveMethod.
	
	self toolbar
		addSpacerToEdge: #top;  
		addButton: saveBtn edge: #top.
	
	revertBtn := UiToolButton new
		autoRaise: true;
		color: Color transparent;  
		buttonStyle: #iconOnly;
		icon: UiFugueIcons arrowReturn180LeftIcon;
		balloonText: 'Discard changed source'.
	self connect: revertBtn signal: #clicked: to: sourceCodeEdit selector: #revertChanges.
	
	self toolbar addButton: revertBtn edge: #top.
	
	deleteBtn := UiToolButton new
		autoRaise: true;
		color: Color transparent;  
		text: '<font color="#FFFFFF">Sure?</font>';
		icon: UiSilkIcons binIcon;
		buttonStyle: #iconOnly;
		balloonText: 'Delete selector'.
	self connect: deleteBtn signal: #clicked: toSelector: #deleteMethod.

	self toolbar
		addSpacerToEdge: #top;
		addButton: deleteBtn edge: #top.

	self updateToolbarButtons.
! !

!SdMethodEditor methodsFor: 'testing' stamp: 'mt 4/6/2011 16:23:36.804'!
isMethodEditor

	^ true! !

!SdMethodEditor methodsFor: 'layout' stamp: 'mt 4/8/2011 10:47:24.544'!
minExtent

	"WORKAROUND: Fix buggy layouting due to #minExtent implementation in some cases."
	self owner ifNotNil: [:o |
		self hResizing == #spaceFill ifTrue: [self width: self owner width].
		self vResizing == #spaceFill ifTrue: [self height: self owner height]].
	
	^ super minExtent! !

!SdMethodEditor methodsFor: 'actions' stamp: 'mt 4/13/2011 15:33:27.075'!
saveMethod

	sourceCodeEdit acceptChanges.! !

!SdMethodEditor methodsFor: 'accessing' stamp: 'mt 3/24/2011 11:24:51.55'!
selector

	^ sourceCodeEdit selector! !

!SdMethodEditor methodsFor: 'accessing' stamp: 'mt 3/25/2011 11:58:34.431'!
selector: aSymbol

	sourceCodeEdit selector: aSymbol.
	self updateLayout.! !

!SdMethodEditor methodsFor: 'accessing' stamp: 'mt 4/12/2011 15:51:33.205'!
selectorText

	| result |
	result := self selector asString asText.
	self sourceCodeEdit hasUnacceptedChanges
		ifTrue: [result addAttribute: TextEmphasis bold].
	"result addAttribute: (TextFontReference toFont: (sourceCodeEdit textMorph textStyle defaultFont))."
	^ result
	! !

!SdMethodEditor methodsFor: 'accessing' stamp: 'mt 4/4/2011 15:10:26.493'!
sourceCodeEdit

	^ sourceCodeEdit! !

!SdMethodEditor methodsFor: 'actions' stamp: 'mt 4/12/2011 17:07:23.466'!
theClass: aClassOrMetaClass selector: aSymbol

	sourceCodeEdit
		theClass: aClassOrMetaClass
		selector: aSymbol.

	self theClass: aClassOrMetaClass.! !

!SdMethodEditor methodsFor: 'updating' stamp: 'mt 4/14/2011 19:26:40.906'!
updateLayout

	toggleBtn checked
		ifTrue: [
			summaryContainer
				disableTableLayout: true;
				hide.
			sourceCodeEdit
				disableTableLayout: false;
				show;
				grabKeyboard.
			separator
				disableTableLayout: false;
				show.
			self
				cellInset: -4; "XXX: HACK!!"
				toolbarVisible: true]
		ifFalse: [
			summaryContainer
				disableTableLayout: false;
				show;
				hResizing: #spaceFill.
			toggleBtn
				text: self selectorText.
"				icon: (UiFugueIcons toggleSmallExpandIcon clippedTo: 9@16)."
			sourceCodeEdit
				disableTableLayout: true;
				topLeft: self topLeft;
				hide.
			separator
				disableTableLayout: true;
				hide.
			self
				cellInset: 0; "XXX: HACK!!"
				toolbarVisible: false].! !

!SdMethodEditor methodsFor: 'updating' stamp: 'mt 4/12/2011 17:44:55.342'!
updateToolbarButtons

	sourceCodeEdit hasUnacceptedChanges
		ifTrue: [revertBtn show. saveBtn show]
		ifFalse: [revertBtn hide. saveBtn hide].! !

Morph subclass: #SdFadingToolbar
	instanceVariableNames: 'leftBar topBar rightBar bottomBar'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sight-Dynamic'!

!SdFadingToolbar methodsFor: 'as yet unclassified' stamp: 'mt 4/7/2011 14:59:47.001'!
addButton: aButton edge: aSymbol

	aSymbol = #left ifTrue: [leftBar addMorphBack: aButton].
	aSymbol = #top ifTrue: [topBar addMorphBack: aButton].
	aSymbol = #right ifTrue: [rightBar addMorphBack: aButton].
	aSymbol = #bottom ifTrue: [bottomBar addMorphBack: aButton].! !

!SdFadingToolbar methodsFor: 'as yet unclassified' stamp: 'mt 4/12/2011 17:29:46.656'!
addSpacerToEdge: aSymbol

	| spacer |
	spacer := Morph new
		color: Color transparent;
		extent: 5@5.
		
	aSymbol = #left ifTrue: [leftBar addMorphBack: spacer].
	aSymbol = #top ifTrue: [topBar addMorphBack: spacer].
	aSymbol = #right ifTrue: [rightBar addMorphBack: spacer].
	aSymbol = #bottom ifTrue: [bottomBar addMorphBack: spacer].! !

!SdFadingToolbar methodsFor: 'as yet unclassified' stamp: 'mt 4/7/2011 14:48:29.386'!
alpha: aFloat
	
	leftBar alpha: aFloat.
	topBar alpha: aFloat.
	rightBar alpha: aFloat.
	bottomBar alpha: aFloat.! !

!SdFadingToolbar methodsFor: 'as yet unclassified' stamp: 'mt 4/12/2011 17:26:45.304'!
fullBounds

	fullBounds ifNil: [
		self updateLayout].
	
	^ super fullBounds! !

!SdFadingToolbar methodsFor: 'as yet unclassified' stamp: 'mt 4/7/2011 16:40:08.924'!
handleMouseMove: anEvent

	self mouseMove: anEvent.! !

!SdFadingToolbar methodsFor: 'as yet unclassified' stamp: 'mt 4/7/2011 17:01:51.113'!
initialize

	super initialize.

	self
		color: Color transparent;
		initializeEventHandler.
	
	leftBar := SdFadingToolbarPanel new orientation: #vertical; wrapDirection: #rightToLeft.  
	topBar := SdFadingToolbarPanel new orientation: #horizontal; wrapDirection: #bottomToTop.
	rightBar := SdFadingToolbarPanel new orientation: #vertical; wrapDirection: #leftToRight.
	bottomBar := SdFadingToolbarPanel new orientation: #horizontal; wrapDirection: #topToBottom.

	self addAllMorphs: {leftBar. topBar. rightBar. bottomBar}.! !

!SdFadingToolbar methodsFor: 'as yet unclassified' stamp: 'mt 4/14/2011 17:16:55.381'!
initializeEventHandler

	self eventHandler: UiEventHandler new.

	self connect: self eventHandler signal: #mouseLeft: toSelector: #alpha: pattern: #(= 0.0).
	self connect: self eventHandler signal: #mouseMoved: toSelector: #updateAlpha:.
	self connect: self eventHandler signal: #mouseEntered: toSelector: #updateAlpha:.! !

!SdFadingToolbar methodsFor: 'as yet unclassified' stamp: 'mt 4/7/2011 17:14:13.551'!
popUpAt: aGlobalPoint

	ActiveWorld submorphs
		select: [:m | m isKindOf: SdFadingToolbar]  
		thenDo: [:m | m delete].
		
	self
		topLeft: aGlobalPoint;
		openInWorld.! !

!SdFadingToolbar methodsFor: 'as yet unclassified' stamp: 'mt 4/7/2011 16:45:39.314'!
rejectsEvent: anEvent

	(self bounds containsPoint: anEvent position)
		ifFalse: [^ super rejectsEvent: anEvent].

	^ (anEvent isMouse and: [
		anEvent isMouseMove
			or: [anEvent isMouseEnter]
			or: [anEvent isMouseLeave]]) not! !

!SdFadingToolbar methodsFor: 'as yet unclassified' stamp: 'mt 4/7/2011 16:55:21.953'!
updateAlpha: aMouseEvent

	(self bounds containsPoint: aMouseEvent position) ifTrue: [
		leftBar alpha: (1.0 - ((aMouseEvent position x - self left max: 0) / (self width / 2)) max: 0.3).
		topBar alpha: (1.0 - ((aMouseEvent position y - self top max: 0) / (self height / 2)) max: 0.3).
		rightBar alpha: (1.0 - ((self right - aMouseEvent position x max: 0) / (self width / 2)) max: 0.3).
		bottomBar alpha: (1.0 - ((self bottom - aMouseEvent position y max: 0) / (self height / 2)) max: 0.3)].! !

!SdFadingToolbar methodsFor: 'as yet unclassified' stamp: 'mt 4/7/2011 16:28:56.17'!
updateLayout
	"Simulate #spaceFill behaviour according to the bar's orientation."
	
	leftBar height: self height.
	topBar width: self width.
	rightBar height: self height.
	bottomBar width: self width.

	leftBar topRight: self topLeft.
	topBar bottomLeft: self topLeft.
	rightBar topLeft: self topRight.
	bottomBar topLeft: self bottomLeft.! !

Morph subclass: #SdFadingToolbarPanel
	instanceVariableNames: 'alpha orientation fadeEnabled'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sight-Dynamic'!

!SdFadingToolbarPanel commentStamp: 'mt 6/21/2010 16:38:29.674' prior: 0!
I just support the XpfCardMorph. You should normally not want to use me directly.

Normally, I contain some buttons and fade them in or out according to my alpha value.!

!SdFadingToolbarPanel methodsFor: 'accessing' stamp: 'mt 6/21/2010 13:35:41.007'!
alpha

	^ alpha ifNil: [alpha := 0.0]! !

!SdFadingToolbarPanel methodsFor: 'accessing' stamp: 'mt 4/7/2011 16:58:02.317'!
alpha: aFloat

	self submorphs
		ifEmpty: [alpha := 0.0]
		ifNotEmpty: [alpha := aFloat].

	self redrawLater.! !

!SdFadingToolbarPanel methodsFor: 'drawing' stamp: 'mt 4/12/2011 17:29:59.446'!
drawOn: aCanvas

	aCanvas
		fillRoundRect: (self submorphBounds outsetBy: 2)
		radius: 5
		fillStyle: (SolidFillStyle color: (Color black alpha: 0.5)).! !

!SdFadingToolbarPanel methodsFor: 'accessing' stamp: 'mt 6/21/2010 16:34:33.892'!
fadeEnabled 
	"Respect the alpha value while drawing?"
	
	^ fadeEnabled ifNil: [fadeEnabled := true]! !

!SdFadingToolbarPanel methodsFor: 'accessing' stamp: 'mt 6/21/2010 16:35:04.687'!
fadeEnabled: aBoolean

	aBoolean = fadeEnabled ifTrue: [^ self].
	
	fadeEnabled := aBoolean.
	self redrawLater.! !

!SdFadingToolbarPanel methodsFor: 'drawing' stamp: 'mt 6/21/2010 16:37:13.905'!
fullDrawOn: aCanvas

	self alpha = 0.0
		ifTrue: [^ self].

	(self alpha < 0.9 and: [self fadeEnabled])
		ifTrue: [^ super fullDrawOn: ((AnimAlphaBlendingCanvas on: aCanvas) alpha: self alpha)]
		ifFalse: [^ super fullDrawOn: aCanvas]! !

!SdFadingToolbarPanel methodsFor: 'initialization' stamp: 'mt 4/12/2011 17:14:58.897'!
initialize

	super initialize.
	
	self initializeEventHandler.
	
	self
		changeTableLayout;
		cellInset: 2;
		layoutInset: 2;
		orientation: #vertical.! !

!SdFadingToolbarPanel methodsFor: 'initialization' stamp: 'mt 4/12/2011 17:59:01.71'!
initializeEventHandler

	self eventHandler: UiEventHandler new.  

	self eventHandler
		grabKeyboardOnMouseEnter: false;  
		setProperty: #handlesMouseDown toValue: [false].

	self connect: self eventHandler signal: #mouseEntered: toSelector: #alpha: pattern: #(= 1.0).
	self connect: self eventHandler signal: #mouseLeft: toSelector: #alpha: pattern: #(= 0.0).
	self connect: self eventHandler signal: #mouseLeft: toSelector: #tryDelete:.! !

!SdFadingToolbarPanel methodsFor: 'accessing' stamp: 'mt 4/7/2011 14:28:28.138'!
orientation

	^ orientation! !

!SdFadingToolbarPanel methodsFor: 'accessing' stamp: 'mt 4/7/2011 14:28:45.026'!
orientation: aSymbol
	"#vertical #horizontal"

	aSymbol = orientation ifTrue: [^ self].

	orientation := aSymbol.
	
	self updateLayout.! !

!SdFadingToolbarPanel methodsFor: 'events-processing' stamp: 'mt 4/11/2011 16:16:45.314'!
rejectsEvent: evt

	(evt isMouse and: [evt isMouseLeave])
		ifTrue: [^ false].
		
	^ self alpha = 0.0 or: [super rejectsEvent: evt]! !

!SdFadingToolbarPanel methodsFor: 'initialization' stamp: 'mt 4/7/2011 17:49:39.259'!
tryDelete: aMouseEvent

	(self owner morphsAt: aMouseEvent position)
		ifEmpty: [self owner delete].! !

!SdFadingToolbarPanel methodsFor: 'layout' stamp: 'mt 4/12/2011 17:33:50.568'!
updateLayout

	self orientation = #vertical
		ifTrue: [self
			width: 22;
			listDirection: #topToBottom].

	self orientation = #horizontal  
		ifTrue: [self
			height: 22;
			listDirection: #leftToRight].! !

BorderedMorph subclass: #SdIDE
	instanceVariableNames: 'codeView addBtn'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sight-Dynamic'!

!SdIDE methodsFor: 'open' stamp: 'mt 4/15/2011 10:26:18.549'!
addClassSelector

	| classEditor resizeHandle |
	classEditor := SdClassEditor new vResizing: #spaceFill.
	resizeHandle := SdResizeHandle new vResizing: #spaceFill.
	resizeHandle target: classEditor.
	
	self connect: classEditor signal: #deleted to: resizeHandle selector: #delete.
	
	codeView widget
		addMorph:	classEditor inFrontOf: addBtn;
		addMorph: resizeHandle inFrontOf: addBtn.! !

!SdIDE methodsFor: 'initialization' stamp: 'mt 4/17/2011 15:11:03.562'!
initialize

	super initialize.
	
	self
		changeTableLayout;
		listDirection: #leftToRight.  
	
	codeView := UiScrollArea new.

	codeView
		hResizing: #spaceFill;
		vResizing: #spaceFill;
		verticalScrollBarPolicy: #alwaysOff;
		horizontalScrollBarPolicy: #asNeeded;  
		widgetResizable: true;
		widget: (Morph new color: (Color gray: 0.85)).
		
	codeView widget
		changeTableLayout;
		listDirection: #leftToRight;
		hResizing: #shrinkWrap;
		cellPositioning: #topLeft.
	
	addBtn := UiToolButton new
		autoRaise: true;
		buttonStyle: #iconOnly;
		icon: UiSilkIcons addIcon;
		balloonText: 'Add class view';
		layoutInset: (2@5 corner: 2@5).
	self connect: addBtn signal: #clicked: toSelector: #addClassSelector.
	codeView widget addMorphBack: addBtn.
	 	
	self
		color: (Color blue muchLighter);
		addMorph: codeView;
		addClassSelector.! !

!SdIDE methodsFor: 'open' stamp: 'mt 4/17/2011 14:48:28.169'!
open

	self extent: 1024@768.

	self openInWindowLabeled: 'Sight Dynamic IDE'.! !

AbstractEvent subclass: #SdMessageEvent
	instanceVariableNames: 'text messageKind'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sight-Dynamic-Events'!

!SdMessageEvent commentStamp: 'mt 4/10/2011 11:17:28.228' prior: 0!
This system event is used to send text messages targeted at classes or messages. Different message kinds allow visual customization.!

!SdMessageEvent class methodsFor: 'accessing' stamp: 'mt 4/10/2011 11:05:10.142'!
changeKind

	^ #Message! !

!SdMessageEvent class methodsFor: 'accessing' stamp: 'mt 4/10/2011 11:06:00.826'!
supportedKinds

	^ {self classKind. self methodKind}! !

!SdMessageEvent class methodsFor: 'instance creation' stamp: 'mt 4/10/2011 11:16:43.624'!
text: stringOrText class: aClass

	^ self
		text: stringOrText
		kind: #information
		class: aClass! !

!SdMessageEvent class methodsFor: 'instance creation' stamp: 'mt 4/10/2011 11:16:26.427'!
text: stringOrText kind: messageKind class: aClass

	| instance |
	instance := self class: aClass.
	instance
		messageKind: messageKind;  
		text: stringOrText.
	^ instance! !

!SdMessageEvent methodsFor: 'testing' stamp: 'mt 4/10/2011 11:18:29.019'!
isError

	^ self messageKind = #error! !

!SdMessageEvent methodsFor: 'testing' stamp: 'mt 4/10/2011 11:18:11.069'!
isInformation

	^ self messageKind = #information! !

!SdMessageEvent methodsFor: 'testing' stamp: 'mt 4/10/2011 11:18:19.13'!
isWarning

	^ self messageKind = #warning! !

!SdMessageEvent methodsFor: 'accessing' stamp: 'mt 4/10/2011 11:17:02.382'!
messageKind
	"The kind of a message can be used to display it differently."
	
	^ messageKind ifNil: [#information]! !

!SdMessageEvent methodsFor: 'accessing' stamp: 'mt 4/10/2011 11:15:53.638'!
messageKind: aSymbol
	"#information, #warning, #error"
	
	messageKind := aSymbol! !

!SdMessageEvent methodsFor: 'printing' stamp: 'mt 4/10/2011 11:06:36.472'!
printEventKindOn: aStream

	aStream nextPutAll: 'Message'! !

!SdMessageEvent methodsFor: 'accessing' stamp: 'mt 4/10/2011 11:09:44.207'!
text

	^ text! !

!SdMessageEvent methodsFor: 'accessing' stamp: 'mt 4/10/2011 11:09:54.777'!
text: aText

	text := aText.! !

!SdMessageEvent methodsFor: 'triggering' stamp: 'mt 4/10/2011 11:24:38.657'!
trigger

	SystemChangeNotifier uniqueInstance trigger: self.! !

UiItemModel subclass: #SdCallgraphModel
	instanceVariableNames: 'root'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sight-Dynamic-Models'!

!SdCallgraphModel commentStamp: 'mt 3/23/2011 14:44:06.545' prior: 0!
| cgt tree |
cgt := SdCallgraphTracer new.
cgt profilingMode: #ms.
cgt selectedCategories: (PackageInfo named: #Widgets) systemCategories.
cgt callgraphModel: SdCallgraph new.
cgt traceSourceCode: 'UiTextEdit new'.
tree := UiTreeView new model: (SdCallgraphModel new root: cgt callgraphModel).
tree openInWorld.!

!SdCallgraphModel methodsFor: 'accessing' stamp: 'mt 3/23/2011 14:40:22.991'!
columnCountFor: aParentModelIndex

	^ 2! !

!SdCallgraphModel methodsFor: 'accessing' stamp: 'mt 3/23/2011 14:40:50.752'!
iconFor: aModelIndex

	(aModelIndex row = 0 or: [aModelIndex column = 0])
		ifTrue: [^ nil].
	
	aModelIndex column = 2
		ifTrue: [^ nil].
	
	^ (aModelIndex valueOfProperty: #snode) ifNotNil: [:snode |
		(snode clientClass inheritsFrom: Morph)
			ifTrue: [snode clientClass widgetClassIcon]
			ifFalse: [nil]]! !

!SdCallgraphModel methodsFor: 'accessing' stamp: 'mt 2/28/2011 11:46:06.509'!
indexRow: aRowNumber column: aColumnNumber parent: aParentModelIndex

	| index snode |
	index := (UiSimpleModelIndex row: aRowNumber column: aColumnNumber)
		model: self;
		parent: aParentModelIndex.
		
	"Header indices."
	(aRowNumber = 0 or: [aColumnNumber = 0])
		ifTrue: [^ index].
		
	snode := aParentModelIndex
		ifNil: [self root rootNode]
		ifNotNil: [aParentModelIndex valueOfProperty: #snode].
		
	index
		setProperty: #snode
		toValue: (snode nextCalls at: aRowNumber).
		
	^ index! !

!SdCallgraphModel methodsFor: 'accessing' stamp: 'mt 2/28/2011 11:41:25.33'!
root

	^ root! !

!SdCallgraphModel methodsFor: 'accessing' stamp: 'mt 2/28/2011 11:41:34.653'!
root: anSNode

	root := anSNode.! !

!SdCallgraphModel methodsFor: 'accessing' stamp: 'mt 3/4/2011 11:51:19.778'!
rowCountFor: aParentModelIndex

	^ (aParentModelIndex
		ifNil: [self root rootNode ifNil: [^0]]
		ifNotNil: [:p | p valueOfProperty: #snode]) nextCalls size! !

!SdCallgraphModel methodsFor: 'accessing' stamp: 'mt 3/23/2011 14:46:41.336'!
textFor: aModelIndex

	aModelIndex row = 0 ifTrue: [
		aModelIndex column = 1 ifTrue: [^ 'CallTree'].
		aModelIndex column = 2 ifTrue: [^ 'Time (ms)']].
	aModelIndex column = 0 ifTrue: [^ ''].
	
	aModelIndex column = 1 ifTrue: [
		^ (aModelIndex valueOfProperty: #snode) ifNotNil: [:snode |
			('{1}\\>\\><b>{2}</b>' format: {snode clientClass name. snode clientSelector}) asHtmlText]].
	
	aModelIndex column = 2 ifTrue: [
		^ (aModelIndex valueOfProperty: #snode) ifNotNil: [:snode |
			(snode profiles at: #ms ifAbsent: ['ms']) allButLast: 2]].! !

UiItemModel subclass: #SdClassTreeModel
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sight-Dynamic-Models'!

!SdClassTreeModel methodsFor: 'counts' stamp: 'mt 4/11/2011 14:22:44.959'!
columnCountFor: aParentModelIndex  

	^ 1! !

!SdClassTreeModel methodsFor: 'model data' stamp: 'mt 4/11/2011 14:29:35.837'!
iconFor: aModelIndex

	^ nil! !

!SdClassTreeModel methodsFor: 'index generation' stamp: 'mt 4/11/2011 14:25:07.643'!
indexRow: aRowNumber column: aColumnNumber parent: aParentModelIndex

	| index |
	index := (UiSimpleModelIndex row: aRowNumber column: aColumnNumber)
		model: self;
		parent: aParentModelIndex.
	
	"Header indices."
	(aRowNumber = 0 or: [aColumnNumber = 0])
		ifTrue: [^ index].
	
	aParentModelIndex ifNil: [
		index setProperty: #package toValue: (PackageOrganizer default packages at: aRowNumber).
		^ index].
	
	(aParentModelIndex includesProperty: #package) ifTrue: [
		index
			setProperty: #systemCategory
			toValue: ((aParentModelIndex valueOfProperty: #package) systemCategories at: aRowNumber).
		index
			setProperty: #systemCategoryPrefix
			toValue: (aParentModelIndex valueOfProperty: #package) systemCategoryPrefix.
		^ index].
	
	(aParentModelIndex includesProperty: #systemCategory) ifTrue: [
		index
			setProperty: #class
			toValue: ((SystemOrganization
				listAtCategoryNamed: (aParentModelIndex valueOfProperty: #systemCategory))
				at: aRowNumber).
		^ index].! !

!SdClassTreeModel methodsFor: 'initialize-release' stamp: 'mt 4/11/2011 14:49:07.869'!
initialize

"	SystemChangeNotifier uniqueInstance
		notify: self ofAllSystemChangesUsing: #processNotification:."! !

!SdClassTreeModel methodsFor: 'notifications' stamp: 'mt 4/11/2011 14:31:07.649'!
processNotification: anEvent

	| pkg pkgIndex sysIndex classIndex |
	(anEvent itemKind ~= #category and: [anEvent itemKind ~= #class])
		ifTrue: [^ self].
	
	pkg := PackageOrganizer default
		packageOfSystemCategory: anEvent itemCategory
		ifNone: [^ self].
	pkgIndex := (UiSimpleModelIndex
		row: (PackageOrganizer default packages indexOf: pkg)
		column: 1)
		model: self;
		setProperty: #package toValue: pkg.
	sysIndex := self
		indexRow: (pkg systemCategories indexOf: anEvent itemCategory)
		column: 1
		parent: pkgIndex.
	classIndex := self
		indexRow: ((SystemOrganization listAtCategoryNamed: anEvent itemCategory) indexOf: anEvent itemClass name)
		column: 1
		parent: sysIndex.
	
	anEvent itemKind = #category ifTrue: [
		anEvent isAdded ifTrue: [
			self rowsInsertedFor: pkgIndex from: sysIndex row to: sysIndex row.
			self rowsChangedFor: nil from: pkgIndex row to: pkgIndex row].
		anEvent isRemoved ifTrue: [
			self rowsRemovedFor: pkgIndex from: sysIndex row to: sysIndex row.
			self rowsChangedFor: nil from: pkgIndex row to: pkgIndex row].
		anEvent isRenamed ifTrue: [
			sysIndex := self
				indexRow: (pkg systemCategories indexOf: anEvent newName)
				column: 1
				parent: pkgIndex.
			self rowsChangedFor: pkgIndex from: sysIndex row to: sysIndex row]].

	anEvent itemKind = #class ifTrue: [
		anEvent isAdded ifTrue: [
			self rowsInsertedFor: sysIndex from: classIndex row to: classIndex row.
			self rowsChangedFor: pkgIndex from: sysIndex row to: sysIndex row].
		anEvent isRenamed ifTrue: [
			self rowsChangedFor: sysIndex from: classIndex row to: classIndex row].
		anEvent isRemoved ifTrue: [
			self rowsRemovedFor: sysIndex from: classIndex row to: classIndex row.
			self rowsChangedFor: pkgIndex from: sysIndex row to: sysIndex row]
		].! !

!SdClassTreeModel methodsFor: 'counts' stamp: 'mt 4/11/2011 14:24:21.631'!
rowCountFor: aParentModelIndex
	
	aParentModelIndex ifNil: [
		^ PackageOrganizer default packages size].
	
	aParentModelIndex column ~= 1 ifTrue: [^ 0].
	
	(aParentModelIndex includesProperty: #package) ifTrue: [
		^ (aParentModelIndex valueOfProperty: #package) systemCategories size].
	
	(aParentModelIndex includesProperty: #systemCategory) ifTrue: [
		^ (SystemOrganization
				listAtCategoryNamed: (aParentModelIndex valueOfProperty: #systemCategory)) size].
		
	^ 0! !

!SdClassTreeModel methodsFor: 'model data' stamp: 'mt 4/11/2011 14:30:10.005'!
textFor: aModelIndex

	| col package systemCategory class |
	col := aModelIndex column.
	package := aModelIndex valueOfProperty: #package ifAbsent: [].
	systemCategory := aModelIndex valueOfProperty: #systemCategory  ifAbsent: [].
	class := aModelIndex valueOfProperty: #class ifAbsent: [].
	
	col ~= 1 ifTrue: [^ nil].
	
	"Horizontal headers."
	aModelIndex row = 0 ifTrue: [^ '<b>Name</b>' asHtmlText].
	
	package ifNotNil: [:p | ^ p name].	
	systemCategory ifNotNil: [:cat | 
		| prefix |
		prefix := aModelIndex valueOfProperty: #systemCategoryPrefix.
			
		cat = prefix ifFalse: [cat := cat allButFirst: prefix size + 1].
		^ cat].
	
	class ifNotNil: [:c | ^ ('<b>', c, '</b>') asHtmlText].
		
	^ nil! !
